using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Collision Model
    /// </summary>
    public class Coll
    {
        #region Constants

        /// <summary>
        /// Size of the coll meta header.
        /// </summary>
        public const int HeaderSize = 52;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private CollLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.6 Shalted: Added some data...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public CollLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class CollLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. ByteArray :: Offset: 0x0 (0)
            /// </summary>
            public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
            private byte[] __unused__;
            /// <summary>
            /// 2. TagBlock :: Offset: 0x14 (20)
            /// </summary>
            public _MaterialsTagBlock Materials { get { return __materials__; } set { __materials__ = value; } }
            private _MaterialsTagBlock __materials__;
            public class _MaterialsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CollLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID MaterialType { get { return __materialType__; } set { __materialType__ = value; } }
                private StringID __materialType__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _MaterialsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _MaterialsTagBlock(CollLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_MaterialsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _MaterialsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 3. TagBlock :: Offset: 0x1C (28)
            /// </summary>
            public _RegionsTagBlock Regions { get { return __regions__; } set { __regions__ = value; } }
            private _RegionsTagBlock __regions__;
            public class _RegionsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CollLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Piece { get { return __piece__; } set { __piece__ = value; } }
                private StringID __piece__;
                /// <summary>
                /// 2. TagBlock :: Offset: 0x4 (4)
                /// </summary>
                public _PermutationsTagBlock Permutations { get { return __permutations__; } set { __permutations__ = value; } }
                private _PermutationsTagBlock __permutations__;
                public class _PermutationsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private CollLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. StringID :: Offset: 0x0 (0)
                    /// </summary>
                    public StringID Permutation { get { return __permutation__; } set { __permutation__ = value; } }
                    private StringID __permutation__;
                    /// <summary>
                    /// 2. TagBlock :: Offset: 0x4 (4)
                    /// </summary>
                    public _BSPTagBlock BSP { get { return __bSP__; } set { __bSP__ = value; } }
                    private _BSPTagBlock __bSP__;
                    public class _BSPTagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private CollLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Float :: Offset: 0x0 (0)
                        /// </summary>
                        public float BSP { get { return __bSP__; } set { __bSP__ = value; } }
                        private float __bSP__;
                        /// <summary>
                        /// 2. TagBlock :: Offset: 0x4 (4)
                        /// </summary>
                        public _N1TagBlock N1 { get { return __n1__; } set { __n1__ = value; } }
                        private _N1TagBlock __n1__;
                        public class _N1TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private CollLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Int16 :: Offset: 0x0 (0)
                            /// </summary>
                            public short Plane { get { return __plane__; } set { __plane__ = value; } }
                            private short __plane__;
                            /// <summary>
                            /// 2. Int16 :: Offset: 0x2 (2)
                            /// </summary>
                            public short FirstChild { get { return __firstChild__; } set { __firstChild__ = value; } }
                            private short __firstChild__;
                            /// <summary>
                            /// 3. Byte :: Offset: 0x4 (4)
                            /// </summary>
                            public byte Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                            private byte __unknown1__;
                            /// <summary>
                            /// 4. Int16 :: Offset: 0x5 (5)
                            /// </summary>
                            public short SecondChild { get { return __secondChild__; } set { __secondChild__ = value; } }
                            private short __secondChild__;
                            /// <summary>
                            /// 5. Byte :: Offset: 0x7 (7)
                            /// </summary>
                            public byte Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                            private byte __unknown2__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _N1TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _N1TagBlock(CollLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_N1TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _N1TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 3. TagBlock :: Offset: 0xC (12)
                        /// </summary>
                        public _Planes1TagBlock Planes1 { get { return __planes1__; } set { __planes1__ = value; } }
                        private _Planes1TagBlock __planes1__;
                        public class _Planes1TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private CollLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Float :: Offset: 0x0 (0)
                            /// </summary>
                            public float I1 { get { return __i1__; } set { __i1__ = value; } }
                            private float __i1__;
                            /// <summary>
                            /// 2. Float :: Offset: 0x4 (4)
                            /// </summary>
                            public float I2 { get { return __i2__; } set { __i2__ = value; } }
                            private float __i2__;
                            /// <summary>
                            /// 3. Float :: Offset: 0x8 (8)
                            /// </summary>
                            public float K { get { return __k__; } set { __k__ = value; } }
                            private float __k__;
                            /// <summary>
                            /// 4. Float :: Offset: 0xC (12)
                            /// </summary>
                            public float D { get { return __d__; } set { __d__ = value; } }
                            private float __d__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Planes1TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Planes1TagBlock(CollLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Planes1TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Planes1TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 4. TagBlock :: Offset: 0x14 (20)
                        /// </summary>
                        public _LeavesTagBlock Leaves { get { return __leaves__; } set { __leaves__ = value; } }
                        private _LeavesTagBlock __leaves__;
                        public class _LeavesTagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private CollLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Flags { get { return __flags__; } set { __flags__ = value; } }
                            private byte __flags__;
                            /// <summary>
                            /// 2. Byte :: Offset: 0x1 (1)
                            /// </summary>
                            public byte N { get { return __n__; } set { __n__ = value; } }
                            private byte __n__;
                            /// <summary>
                            /// 3. Int16 :: Offset: 0x2 (2)
                            /// </summary>
                            public short ReferenceCount { get { return __referenceCount__; } set { __referenceCount__ = value; } }
                            private short __referenceCount__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _LeavesTagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _LeavesTagBlock(CollLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_LeavesTagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _LeavesTagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 5. TagBlock :: Offset: 0x1C (28)
                        /// </summary>
                        public _N2TagBlock N2 { get { return __n2__; } set { __n2__ = value; } }
                        private _N2TagBlock __n2__;
                        public class _N2TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private CollLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Int16 :: Offset: 0x0 (0)
                            /// </summary>
                            public short Plane { get { return __plane__; } set { __plane__ = value; } }
                            private short __plane__;
                            /// <summary>
                            /// 2. Int16 :: Offset: 0x2 (2)
                            /// </summary>
                            public short Node { get { return __node__; } set { __node__ = value; } }
                            private short __node__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _N2TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _N2TagBlock(CollLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_N2TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _N2TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 6. TagBlock :: Offset: 0x24 (36)
                        /// </summary>
                        public _Planes2TagBlock Planes2 { get { return __planes2__; } set { __planes2__ = value; } }
                        private _Planes2TagBlock __planes2__;
                        public class _Planes2TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private CollLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Float :: Offset: 0x0 (0)
                            /// </summary>
                            public float PlaneI { get { return __planeI__; } set { __planeI__ = value; } }
                            private float __planeI__;
                            /// <summary>
                            /// 2. Float :: Offset: 0x4 (4)
                            /// </summary>
                            public float PlaneJ { get { return __planeJ__; } set { __planeJ__ = value; } }
                            private float __planeJ__;
                            /// <summary>
                            /// 3. Float :: Offset: 0x8 (8)
                            /// </summary>
                            public float PlaneK { get { return __planeK__; } set { __planeK__ = value; } }
                            private float __planeK__;
                            /// <summary>
                            /// 4. Int16 :: Offset: 0xC (12)
                            /// </summary>
                            public short LeftChild { get { return __leftChild__; } set { __leftChild__ = value; } }
                            private short __leftChild__;
                            /// <summary>
                            /// 5. Int16 :: Offset: 0xE (14)
                            /// </summary>
                            public short RightChild { get { return __rightChild__; } set { __rightChild__ = value; } }
                            private short __rightChild__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Planes2TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Planes2TagBlock(CollLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Planes2TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Planes2TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 7. TagBlock :: Offset: 0x2C (44)
                        /// </summary>
                        public _SurfacesTagBlock Surfaces { get { return __surfaces__; } set { __surfaces__ = value; } }
                        private _SurfacesTagBlock __surfaces__;
                        public class _SurfacesTagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private CollLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Int16 :: Offset: 0x0 (0)
                            /// </summary>
                            public short Plane { get { return __plane__; } set { __plane__ = value; } }
                            private short __plane__;
                            /// <summary>
                            /// 2. Int16 :: Offset: 0x2 (2)
                            /// </summary>
                            public short FirstEdge { get { return __firstEdge__; } set { __firstEdge__ = value; } }
                            private short __firstEdge__;
                            /// <summary>
                            /// 3. Byte :: Offset: 0x4 (4)
                            /// </summary>
                            public byte Flags { get { return __flags__; } set { __flags__ = value; } }
                            private byte __flags__;
                            /// <summary>
                            /// 4. Byte :: Offset: 0x5 (5)
                            /// </summary>
                            public byte BreakableSurface { get { return __breakableSurface__; } set { __breakableSurface__ = value; } }
                            private byte __breakableSurface__;
                            /// <summary>
                            /// 5. Int16 :: Offset: 0x6 (6)
                            /// </summary>
                            public short Material { get { return __material__; } set { __material__ = value; } }
                            private short __material__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _SurfacesTagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _SurfacesTagBlock(CollLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_SurfacesTagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _SurfacesTagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 8. TagBlock :: Offset: 0x34 (52)
                        /// </summary>
                        public _EdgeBufferTagBlock EdgeBuffer { get { return __edgeBuffer__; } set { __edgeBuffer__ = value; } }
                        private _EdgeBufferTagBlock __edgeBuffer__;
                        public class _EdgeBufferTagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private CollLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Int16 :: Offset: 0x0 (0)
                            /// </summary>
                            public short StartVertex { get { return __startVertex__; } set { __startVertex__ = value; } }
                            private short __startVertex__;
                            /// <summary>
                            /// 2. Int16 :: Offset: 0x2 (2)
                            /// </summary>
                            public short EndVertex { get { return __endVertex__; } set { __endVertex__ = value; } }
                            private short __endVertex__;
                            /// <summary>
                            /// 3. Int16 :: Offset: 0x4 (4)
                            /// </summary>
                            public short ForwardEdge { get { return __forwardEdge__; } set { __forwardEdge__ = value; } }
                            private short __forwardEdge__;
                            /// <summary>
                            /// 4. Int16 :: Offset: 0x6 (6)
                            /// </summary>
                            public short ReverseEdge { get { return __reverseEdge__; } set { __reverseEdge__ = value; } }
                            private short __reverseEdge__;
                            /// <summary>
                            /// 5. Int16 :: Offset: 0x8 (8)
                            /// </summary>
                            public short LeftSurface { get { return __leftSurface__; } set { __leftSurface__ = value; } }
                            private short __leftSurface__;
                            /// <summary>
                            /// 6. Int16 :: Offset: 0xA (10)
                            /// </summary>
                            public short RightSurface { get { return __rightSurface__; } set { __rightSurface__ = value; } }
                            private short __rightSurface__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _EdgeBufferTagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _EdgeBufferTagBlock(CollLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_EdgeBufferTagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _EdgeBufferTagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 9. TagBlock :: Offset: 0x3C (60)
                        /// </summary>
                        public _VerticesTagBlock Vertices { get { return __vertices__; } set { __vertices__ = value; } }
                        private _VerticesTagBlock __vertices__;
                        public class _VerticesTagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private CollLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Float :: Offset: 0x0 (0)
                            /// </summary>
                            public float X { get { return __x__; } set { __x__ = value; } }
                            private float __x__;
                            /// <summary>
                            /// 2. Float :: Offset: 0x4 (4)
                            /// </summary>
                            public float Y { get { return __y__; } set { __y__ = value; } }
                            private float __y__;
                            /// <summary>
                            /// 3. Float :: Offset: 0x8 (8)
                            /// </summary>
                            public float Z { get { return __z__; } set { __z__ = value; } }
                            private float __z__;
                            /// <summary>
                            /// 4. Int32 :: Offset: 0xC (12)
                            /// </summary>
                            public int FirstEdge { get { return __firstEdge__; } set { __firstEdge__ = value; } }
                            private int __firstEdge__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _VerticesTagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _VerticesTagBlock(CollLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_VerticesTagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _VerticesTagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _BSPTagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _BSPTagBlock(CollLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                            N1 = new _N1TagBlock(layout);
                            Planes1 = new _Planes1TagBlock(layout);
                            Leaves = new _LeavesTagBlock(layout);
                            N2 = new _N2TagBlock(layout);
                            Planes2 = new _Planes2TagBlock(layout);
                            Surfaces = new _SurfacesTagBlock(layout);
                            EdgeBuffer = new _EdgeBufferTagBlock(layout);
                            Vertices = new _VerticesTagBlock(layout);
                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_BSPTagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _BSPTagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }
                    /// <summary>
                    /// 3. TagBlock :: Offset: 0xC (12)
                    /// </summary>
                    public _UnknownTagBlock Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                    private _UnknownTagBlock __unknown__;
                    public class _UnknownTagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private CollLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Float :: Offset: 0x0 (0)
                        /// </summary>
                        public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                        private float __unknown1__;
                        /// <summary>
                        /// 2. Float :: Offset: 0x4 (4)
                        /// </summary>
                        public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                        private float __unknown2__;
                        /// <summary>
                        /// 3. ByteArray :: Offset: 0x8 (8)
                        /// </summary>
                        public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                        private byte[] __unused1__;
                        /// <summary>
                        /// 4. Float :: Offset: 0xC (12)
                        /// </summary>
                        public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                        private float __unknown3__;
                        /// <summary>
                        /// 5. Float :: Offset: 0x10 (16)
                        /// </summary>
                        public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                        private float __unknown4__;
                        /// <summary>
                        /// 6. Float :: Offset: 0x14 (20)
                        /// </summary>
                        public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                        private float __unknown5__;
                        /// <summary>
                        /// 7. Float :: Offset: 0x18 (24)
                        /// </summary>
                        public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                        private float __unknown6__;
                        /// <summary>
                        /// 8. ByteArray :: Offset: 0x1C (28)
                        /// </summary>
                        public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                        private byte[] __unused2__;
                        /// <summary>
                        /// 9. Float :: Offset: 0x20 (32)
                        /// </summary>
                        public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                        private float __unknown7__;
                        /// <summary>
                        /// 10. Float :: Offset: 0x24 (36)
                        /// </summary>
                        public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                        private float __unknown8__;
                        /// <summary>
                        /// 11. Float :: Offset: 0x28 (40)
                        /// </summary>
                        public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                        private float __unknown9__;
                        /// <summary>
                        /// 12. ByteArray :: Offset: 0x2C (44)
                        /// </summary>
                        public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                        private byte[] __unused3__;
                        /// <summary>
                        /// 13. ID :: Offset: 0x30 (48)
                        /// </summary>
                        public LoneID Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                        private LoneID __unknown10__;
                        /// <summary>
                        /// 14. ByteArray :: Offset: 0x34 (52)
                        /// </summary>
                        public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
                        private byte[] __unused4__;
                        /// <summary>
                        /// 15. Float :: Offset: 0x40 (64)
                        /// </summary>
                        public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                        private float __unknown11__;
                        /// <summary>
                        /// 16. Float :: Offset: 0x44 (68)
                        /// </summary>
                        public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                        private float __unknown12__;
                        /// <summary>
                        /// 17. ByteArray :: Offset: 0x48 (72)
                        /// </summary>
                        public byte[] Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
                        private byte[] __unused5__;
                        /// <summary>
                        /// 18. Float :: Offset: 0x4C (76)
                        /// </summary>
                        public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                        private float __unknown13__;
                        /// <summary>
                        /// 19. Float :: Offset: 0x50 (80)
                        /// </summary>
                        public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                        private float __unknown14__;
                        /// <summary>
                        /// 20. Float :: Offset: 0x54 (84)
                        /// </summary>
                        public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                        private float __unknown15__;
                        /// <summary>
                        /// 21. ByteArray :: Offset: 0x58 (88)
                        /// </summary>
                        public byte[] Unused6 { get { return __unused6__; } set { __unused6__ = value; } }
                        private byte[] __unused6__;
                        /// <summary>
                        /// 22. Float :: Offset: 0x5C (92)
                        /// </summary>
                        public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                        private float __unknown16__;
                        /// <summary>
                        /// 23. Float :: Offset: 0x60 (96)
                        /// </summary>
                        public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                        private float __unknown17__;
                        /// <summary>
                        /// 24. TagBlock :: Offset: 0x64 (100)
                        /// </summary>
                        public _Unknown18TagBlock Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                        private _Unknown18TagBlock __unknown18__;
                        public class _Unknown18TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private CollLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private byte __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown18TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown18TagBlock(CollLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown18TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown18TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 25. ByteArray :: Offset: 0x6C (108)
                        /// </summary>
                        public byte[] Unused7 { get { return __unused7__; } set { __unused7__ = value; } }
                        private byte[] __unused7__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _UnknownTagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _UnknownTagBlock(CollLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                            Unknown18 = new _Unknown18TagBlock(layout);
                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_UnknownTagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _UnknownTagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _PermutationsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _PermutationsTagBlock(CollLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        BSP = new _BSPTagBlock(layout);
                        Unknown = new _UnknownTagBlock(layout);
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_PermutationsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _PermutationsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _RegionsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _RegionsTagBlock(CollLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Permutations = new _PermutationsTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_RegionsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _RegionsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 4. TagBlock :: Offset: 0x24 (36)
            /// </summary>
            public _PathFindingSpheresTagBlock PathFindingSpheres { get { return __pathFindingSpheres__; } set { __pathFindingSpheres__ = value; } }
            private _PathFindingSpheresTagBlock __pathFindingSpheres__;
            public class _PathFindingSpheresTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CollLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Int32 :: Offset: 0x0 (0)
                /// </summary>
                public int NodeChunk { get { return __nodeChunk__; } set { __nodeChunk__ = value; } }
                private int __nodeChunk__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float CenterPointX { get { return __centerPointX__; } set { __centerPointX__ = value; } }
                private float __centerPointX__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float CenterPointY { get { return __centerPointY__; } set { __centerPointY__ = value; } }
                private float __centerPointY__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float CenterPointZ { get { return __centerPointZ__; } set { __centerPointZ__ = value; } }
                private float __centerPointZ__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Radius { get { return __radius__; } set { __radius__ = value; } }
                private float __radius__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _PathFindingSpheresTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _PathFindingSpheresTagBlock(CollLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_PathFindingSpheresTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _PathFindingSpheresTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 5. TagBlock :: Offset: 0x2C (44)
            /// </summary>
            public _NodesTagBlock Nodes { get { return __nodes__; } set { __nodes__ = value; } }
            private _NodesTagBlock __nodes__;
            public class _NodesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CollLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CollLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Node { get { return __node__; } set { __node__ = value; } }
                private StringID __node__;
                /// <summary>
                /// 2. Int16 :: Offset: 0x4 (4)
                /// </summary>
                public short Parent { get { return __parent__; } set { __parent__ = value; } }
                private short __parent__;
                /// <summary>
                /// 3. Int16 :: Offset: 0x6 (6)
                /// </summary>
                public short Child { get { return __child__; } set { __child__ = value; } }
                private short __child__;
                /// <summary>
                /// 4. Int16 :: Offset: 0x8 (8)
                /// </summary>
                public short NextSibling { get { return __nextSibling__; } set { __nextSibling__ = value; } }
                private short __nextSibling__;
                /// <summary>
                /// 5. Int16 :: Offset: 0xA (10)
                /// </summary>
                public short FinalSiblingPossibly { get { return __finalSiblingPossibly__; } set { __finalSiblingPossibly__ = value; } }
                private short __finalSiblingPossibly__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _NodesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _NodesTagBlock(CollLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_NodesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _NodesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }

            #endregion

            /// <summary>
            /// Initializes a new instance of the CollLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public CollLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Materials = new _MaterialsTagBlock(this);
                Regions = new _RegionsTagBlock(this);
                PathFindingSpheres = new _PathFindingSpheresTagBlock(this);
                Nodes = new _NodesTagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Unused = br.ReadBytes(20);
                Materials._Initialize(br);
                Regions._Initialize(br);
                PathFindingSpheres._Initialize(br);
                Nodes._Initialize(br);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Coll class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Coll(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new CollLayout(map, identity);
        }

        #region Specific Methods of coll

        #endregion
    }
}
