using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Sound Environment
    /// </summary>
    public class Snde
    {
        #region Constants

        /// <summary>
        /// Size of the snde meta header.
        /// </summary>
        public const int HeaderSize = 72;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private SndeLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v1.0 Shalted: Hehe, Finished.
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public SndeLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class SndeLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. ByteArray :: Offset: 0x0 (0)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 2. Float :: Offset: 0x8 (8)
            /// </summary>
            public float Priority { get { return __priority__; } set { __priority__ = value; } }
            private float __priority__;
            /// <summary>
            /// 3. Float :: Offset: 0xC (12)
            /// </summary>
            public float RoomIntensity_dB { get { return __roomIntensity_dB__; } set { __roomIntensity_dB__ = value; } }
            private float __roomIntensity_dB__;
            /// <summary>
            /// 4. Float :: Offset: 0x10 (16)
            /// </summary>
            public float RoomIntensityHf_dB { get { return __roomIntensityHf_dB__; } set { __roomIntensityHf_dB__ = value; } }
            private float __roomIntensityHf_dB__;
            /// <summary>
            /// 5. Float :: Offset: 0x14 (20)
            /// </summary>
            public float RoomRolloff_ { get { return __roomRolloff___; } set { __roomRolloff___ = value; } }
            private float __roomRolloff___;
            /// <summary>
            /// 6. Float :: Offset: 0x18 (24)
            /// </summary>
            public float DecayTime_ { get { return __decayTime___; } set { __decayTime___ = value; } }
            private float __decayTime___;
            /// <summary>
            /// 7. Float :: Offset: 0x1C (28)
            /// </summary>
            public float DecayHfRattion_ { get { return __decayHfRattion___; } set { __decayHfRattion___ = value; } }
            private float __decayHfRattion___;
            /// <summary>
            /// 8. Float :: Offset: 0x20 (32)
            /// </summary>
            public float ReflectionsDelay_ { get { return __reflectionsDelay___; } set { __reflectionsDelay___ = value; } }
            private float __reflectionsDelay___;
            /// <summary>
            /// 9. Float :: Offset: 0x24 (36)
            /// </summary>
            public float ReverbIntensity_dB { get { return __reverbIntensity_dB__; } set { __reverbIntensity_dB__ = value; } }
            private float __reverbIntensity_dB__;
            /// <summary>
            /// 10. Float :: Offset: 0x28 (40)
            /// </summary>
            public float ReverbDelay_ { get { return __reverbDelay___; } set { __reverbDelay___ = value; } }
            private float __reverbDelay___;
            /// <summary>
            /// 11. Float :: Offset: 0x2C (44)
            /// </summary>
            public float Diffusion { get { return __diffusion__; } set { __diffusion__ = value; } }
            private float __diffusion__;
            /// <summary>
            /// 12. Float :: Offset: 0x30 (48)
            /// </summary>
            public float Density { get { return __density__; } set { __density__ = value; } }
            private float __density__;
            /// <summary>
            /// 13. Float :: Offset: 0x34 (52)
            /// </summary>
            public float HfReference_ { get { return __hfReference___; } set { __hfReference___ = value; } }
            private float __hfReference___;
            /// <summary>
            /// 14. ByteArray :: Offset: 0x38 (56)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the SndeLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public SndeLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Unused1 = br.ReadBytes(8);
                Priority = br.ReadSingle();
                RoomIntensity_dB = br.ReadSingle();
                RoomIntensityHf_dB = br.ReadSingle();
                RoomRolloff_ = br.ReadSingle();
                DecayTime_ = br.ReadSingle();
                DecayHfRattion_ = br.ReadSingle();
                ReflectionsDelay_ = br.ReadSingle();
                ReverbIntensity_dB = br.ReadSingle();
                ReverbDelay_ = br.ReadSingle();
                Diffusion = br.ReadSingle();
                Density = br.ReadSingle();
                HfReference_ = br.ReadSingle();
                Unused2 = br.ReadBytes(16);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Snde class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Snde(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new SndeLayout(map, identity);
        }

        #region Specific Methods of snde

        #endregion
    }
}
