using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.IO;
using System.Extended;
using H2UtilityToolkit;
using H2UtilityToolkit.Misc;
using Microsoft.DirectX.Direct3D;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Bitmap
    /// </summary>
    public class Bitm
    {
        #region Instance Data

        #region Constants

        /// <summary>
        /// Size of the bitm meta header.
        /// </summary>
        public const int HeaderSize = 76;

        #endregion

        #region Static

        [DllImport("kernel32.dll")]
        static extern void RtlMoveMemory(IntPtr Pointer, byte[] Buffer, int Count);

        private static RGBA_ByteColor[] H2Palette;
        private static bool H2PaletteLoaded;

        #endregion

        private Map __map__;
        private int __identity__;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return __map__; } set { __map__ = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return __identity__; } set { __identity__ = value; } }

        #endregion

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Bitm class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Bitm(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
        }

        #region Specific Methods of bitm

        #region Append

        /// <summary>
        /// 
        /// </summary>
        public void AppendBitmapRaw()
        {
            FileStream fs = new FileStream(Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
            AppendBitmapRaw(fs);
            fs.Dispose();
        }

        /// <summary>
        /// Internalises, also internal bitmaps.
        /// </summary>
        /// <param name="fs"></param>
        public void AppendBitmapRaw(FileStream fs)
        {
            int tagIndx = Map.Tags.IndexOf(Identity);
            if (tagIndx == -1) return;

            BitReader br = new BitReader(fs, Endian.Little);
            BitWriter bw = new BitWriter(fs, Endian.Little);

            br.BaseStream.Position = Map.Header.IndexOffset;
            byte[] buffer = br.ReadBytes((int)(br.BaseStream.Length - br.BaseStream.Position));

            br.BaseStream.Position = Map.Tags.TagsList[tagIndx].MetaOffset + 68;

            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            List<RawInfo> rawInfoList = new List<RawInfo>();
            for (int x = 0; x < cc; x++)
            {
                for (int y = 0; y < 6; y++)
                {
                    RawInfo rawInfo = new RawInfo();
                    br.BaseStream.Position = tr + (x * 116) + 28 + (y * 4);
                    rawInfo.RawOffsetDeclaration = (int)br.BaseStream.Position - Map.Tags.TagsList[tagIndx].MetaOffset;

                    rawInfo.RawOffset = br.ReadInt32();
                    br.BaseStream.Position += 20;
                    rawInfo.RawSize = br.ReadInt32();

                    if (rawInfo.RawOffset != -1 && rawInfo.RawSize != 0)
                    {
                        Map.Misc.ParsePointer(ref rawInfo.RawOffset,
                            out rawInfo.RawLocation);

                        rawInfoList.Add(rawInfo);
                    }
                }
            }

            br.BaseStream.Position = Map.Header.IndexOffset;
            int shift = 0;
            for (int x = 0; x < rawInfoList.Count; x++)
            {
                rawInfoList[x].NewRawOffset = (int)br.BaseStream.Position;
                if (rawInfoList[x].RawLocation == RawLocation.Internal)
                {
                    br.BaseStream.Position = rawInfoList[x].RawOffset;
                    byte[] imgBuffer = br.ReadBytes(rawInfoList[x].RawSize);

                    bw.BaseStream.Position = rawInfoList[x].NewRawOffset;
                    bw.Write(imgBuffer);
                }
                else
                {
                    FileStream rawFs = Map.Prefs.GetRawStream(rawInfoList[x].RawLocation);
                    if (rawFs == null)
                    {
                        bw.Write(new byte[rawInfoList[x].RawSize]);
                    }
                    else
                    {
                        BitReader rawBr = new BitReader(rawFs, Endian.Little);
                        rawBr.BaseStream.Position = rawInfoList[x].RawOffset;
                        bw.Write(rawBr.ReadBytes(rawInfoList[x].RawSize));
                        rawBr.Close();
                    }
                }

                shift += rawInfoList[x].RawSize;
                rawInfoList[x].RawOffset = rawInfoList[x].NewRawOffset;

                int padding = 512 - (rawInfoList[x].RawSize % 512);
                if (padding != 512)
                {
                    bw.Write(new byte[padding]);
                    shift += padding;
                }
            }

            Map.Header.IndexOffset = (int)bw.BaseStream.Position;
            bw.Write(buffer);

            //--[Header]
            bw.BaseStream.Position = 16;
            bw.Write(Map.Header.IndexOffset);

            //--[Shift]
            Map.Index.ShiftValuesInApp(shift);
            for (int x = 0; x < Map.Tags.TagsList.Length; x++)
            {
                if (Map.Tags.TagsList[x].Type != "sbsp" && Map.Tags.TagsList[x].Type != "ltmp")
                    Map.Tags.TagsList[x].MetaOffset += shift;
            }

            //--[Meta]
            for (int x = 0; x < rawInfoList.Count; x++)
            {
                br.BaseStream.Position = Map.Tags.TagsList[tagIndx].MetaOffset + rawInfoList[x].RawOffsetDeclaration;
                bw.Write(rawInfoList[x].RawOffset);
            }

            //--[End]
            bw.BaseStream.Position =
                Map.Tags.TagsList[Map.Tags.TagsList.Length - 1].MetaOffset +
                Map.Tags.TagsList[Map.Tags.TagsList.Length - 1].MetaSize;

            int newPaddingLen = 512 - ((int)bw.BaseStream.Position % 512);
            newPaddingLen += (newPaddingLen == 512 ? 512 : 1024);

            int eof = (int)bw.BaseStream.Position + newPaddingLen;
            byte cd = 205;
            while (bw.BaseStream.Position < eof) bw.Write(cd);
            bw.BaseStream.SetLength(eof);

            //--[Header]
            bw.BaseStream.Position = 8;
            bw.Write(eof);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mapStream"></param>
        /// <param name="imgBuffer"></param>
        /// <param name="submapIndx"></param>
        /// <param name="LOD"></param>
        private int AppendBitmapRaw(FileStream fs, byte[] imgBuffer, int submapIndx, int LOD)
        {
            int tagIndx = Map.Tags.IndexOf(Identity);
            if (tagIndx == -1) return 0;

            BitReader br = new BitReader(fs, Endian.Little);
            BitWriter bw = new BitWriter(fs, Endian.Little);

            br.BaseStream.Position = Map.Header.IndexOffset;
            byte[] buffer = br.ReadBytes((int)(br.BaseStream.Length - br.BaseStream.Position));

            int newRawOffset = Map.Header.IndexOffset;
            br.BaseStream.Position = newRawOffset;

            bw.Write(imgBuffer);
            int shift = imgBuffer.Length;

            int newPadding = 512 - (imgBuffer.Length % 512);
            if (newPadding != 512)
            {
                bw.Write(new byte[newPadding]);
                shift += newPadding;
            }

            Map.Header.IndexOffset = (int)bw.BaseStream.Position;
            bw.Write(buffer);

            //--[Header]
            bw.BaseStream.Position = 16;
            bw.Write(Map.Header.IndexOffset);

            //--[Shift]
            Map.Index.ShiftValuesInApp(shift);
            for (int x = 0; x < Map.Tags.TagsList.Length; x++)
            {
                if (Map.Tags.TagsList[x].Type != "sbsp" && Map.Tags.TagsList[x].Type != "ltmp")
                    Map.Tags.TagsList[x].MetaOffset += shift;
            }

            //--[Meta]
            br.BaseStream.Position = Map.Tags.TagsList[tagIndx].MetaOffset + 68;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            bw.BaseStream.Position = tr + (submapIndx * 116) + 28 + (LOD * 4);
            bw.Write(newRawOffset);
            bw.BaseStream.Position += 20;
            bw.Write(imgBuffer.Length);

            //--[End]
            bw.BaseStream.Position =
                Map.Tags.TagsList[Map.Tags.TagsList.Length - 1].MetaOffset +
                Map.Tags.TagsList[Map.Tags.TagsList.Length - 1].MetaSize;

            int newPaddingLen = 512 - ((int)bw.BaseStream.Position % 512);
            newPaddingLen += (newPaddingLen == 512 ? 512 : 1024);

            int eof = (int)bw.BaseStream.Position + newPaddingLen;
            byte cd = 205;
            while (bw.BaseStream.Position < eof) bw.Write(cd);
            bw.BaseStream.SetLength(eof);

            //--[Header]
            bw.BaseStream.Position = 8;
            bw.Write(eof);

            return shift;
        }

        #endregion

        #region Get

        public Bitmap GetBitmap(int submapIndx, int LOD, int paletteIndx, int sbspIndx)
        {
            BitReader br = new BitReader(Map.FileName, Endian.Little);
            Bitmap bitm = GetBitmap(br, submapIndx, LOD, paletteIndx, sbspIndx);
            br.Close();

            return bitm;
        }

        public Bitmap GetBitmap(BitReader br, int submapIndx, int LOD, int paletteIndx, int sbspIndx)
        {
            int tagIndx = Map.Tags.IndexOf(Identity);
            if (tagIndx == -1) return null;

            br.BaseStream.Position = Map.Tags.TagsList[tagIndx].MetaOffset + 68;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            if (submapIndx >= cc || tr <= 0) return null;
            br.BaseStream.Position = tr + (submapIndx * 116);

            BitmapInfo BitmInfo = new BitmapInfo(br);
            br.BaseStream.Position += 4 + (LOD * 4);

            int RawOffset = br.ReadInt32();
            br.BaseStream.Position += 20;
            int RawSize = br.ReadInt32();

            if (RawOffset == -1 || RawSize == 0)
            {
                return null;
            }
            else
            {
                RawLocation rawLoc;
                Map.Misc.ParsePointer(ref RawOffset, out rawLoc);

                if (rawLoc == RawLocation.Internal)
                {
                    br.BaseStream.Position = RawOffset;
                    return DecodeForBitmap(Map, br.ReadBytes(RawSize), BitmInfo, paletteIndx, sbspIndx);
                }
                else
                {
                    FileStream fs = Map.Prefs.GetRawStream(rawLoc);
                    if (fs == null) return null;
                    else
                    {
                        BitReader rawBr = new BitReader(fs, Endian.Little);
                        rawBr.BaseStream.Position = RawOffset;
                        Bitmap bitm = DecodeForBitmap(__map__, rawBr.ReadBytes(RawSize), BitmInfo, paletteIndx, sbspIndx);
                        rawBr.Close();

                        return bitm;
                    }
                }
            }
        }

        #endregion

        #region DDS Methods

        public void ExtractAsDDS(int submapIndx, int LOD, int paletteIndx, int sbspIndx, string fileName)
        {
            int tagIndx = Map.Tags.IndexOf(Identity);
            if (tagIndx == -1) return;

            BitReader br = new BitReader(Map.FileName, Endian.Little);

            br.BaseStream.Position = Map.Tags.TagsList[tagIndx].MetaOffset + 68;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            if (submapIndx >= cc || tr <= 0)
            {
                br.Close();
                return;
            }

            br.BaseStream.Position = tr + (submapIndx * 116);

            BitmapInfo BitmInfo = new BitmapInfo(br);
            br.BaseStream.Position += 4 + (LOD * 4);

            int RawOffset = br.ReadInt32();
            br.BaseStream.Position += 20;
            int RawSize = br.ReadInt32();

            byte[] ddsBuffer = null;
            if (RawOffset != -1 && RawSize != 0)
            {
                RawLocation rawLoc;
                Map.Misc.ParsePointer(ref RawOffset, out rawLoc);

                if (rawLoc == RawLocation.Internal)
                {
                    br.BaseStream.Position = RawOffset;
                    ddsBuffer = DecodeForDDS(Map, br.ReadBytes(RawSize), BitmInfo, paletteIndx, sbspIndx);
                }
                else
                {
                    FileStream fs = Map.Prefs.GetRawStream(rawLoc);
                    if (fs != null)
                    {
                        BitReader rawBr = new BitReader(fs, Endian.Little);
                        rawBr.BaseStream.Position = RawOffset;
                        ddsBuffer = DecodeForDDS(Map, rawBr.ReadBytes(RawSize), BitmInfo, paletteIndx, sbspIndx);
                        rawBr.Close();
                    }
                }
            }
            br.Close();
            if (ddsBuffer == null) return;

            BitWriter bw = new BitWriter(new FileStream(fileName, FileMode.Create,
                FileAccess.ReadWrite, FileShare.ReadWrite), Endian.Little);

            DDS_Header dds_Header = new DDS_Header();
            dds_Header.Generate(BitmInfo);
            for (int x = 0; x < LOD; x++)
            {
                dds_Header.dds_SurfaceDesc.width /= 2;
                dds_Header.dds_SurfaceDesc.height /= 2;
            }
            dds_Header.WriteStruct(bw);
            bw.Write(ddsBuffer);

            bw.Close();
        }

        public void InjectDDS(BitmapInfo bInfo)
        {
            int tagIndx = Map.Tags.IndexOf(Identity);
            if (tagIndx == -1) return;

            BitReader br = new BitReader(Map.FileName, Endian.Little);
            BitWriter bw = new BitWriter(br.BaseStream, Endian.Little);

            br.BaseStream.Position = Map.Tags.TagsList[tagIndx].MetaOffset + 68;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            // We only concentrate on the first submap
            int submapIndx = 0;

            ushort width = bInfo.Width;
            ushort height = bInfo.Height;

            using (System.Windows.Forms.OpenFileDialog o = new System.Windows.Forms.OpenFileDialog())
            {
                o.Filter = "DDS (*.dds)|*.dds";

                // LOD's
                for (int y = 0; y < 6; y++)
                {
                    br.BaseStream.Position = tr + (submapIndx * 116) + 28 + (y * 4);

                    int rawOffset = br.ReadInt32();
                    br.BaseStream.Position += 20;
                    int rawSize = br.ReadInt32();

                    if (rawOffset != -1 && rawSize != 0)
                    {
                        o.Title = string.Format("Detail Level {0}", y + 1);
                        o.FileName = string.Empty;
                        if (o.ShowDialog() != System.Windows.Forms.DialogResult.Cancel)
                        {
                            BitReader imgBr = new BitReader(o.FileName, Endian.Little);

                            if (y == 0)
                            {
                                // Read DDS header
                                DDS_Header dds_Header = new DDS_Header();
                                dds_Header.ReadStruct(imgBr);
                                dds_Header.FixBitmapInfo(ref bInfo);
                                width = bInfo.Width;
                                height = bInfo.Height;

                                // Write header
                                br.BaseStream.Position = tr + (submapIndx * 116);
                                bInfo.WriteInfo(bw);
                            }
                            else
                            {
                                bInfo.Width /= 2;
                                bInfo.Height /= 2;
                            }

                            // Read DDS body
                            imgBr.BaseStream.Position = 128;
                            byte[] imgBuffer = EncodeForMap(imgBr.ReadBytes((int)imgBr.BaseStream.Length - 128), bInfo, 0, 0);
                            imgBr.Close();

                            // Write to map
                            int diff = imgBuffer.Length - rawSize;

                            RawLocation rawLoc;
                            Map.Misc.ParsePointer(ref rawOffset, out rawLoc);

                            if (rawLoc == RawLocation.Internal)
                            {
                                if (diff == 0) // Correct Size
                                {
                                    br.BaseStream.Position = rawOffset;
                                    bw.Write(imgBuffer);
                                }
                                else if (diff < 0) // Smaller
                                {
                                    int shift = InjectSmallerBiggerBitmap((br.BaseStream as FileStream), imgBuffer, submapIndx, y);
                                    tr += shift;
                                }
                                else // Bigger
                                {
                                    int shift = InjectSmallerBiggerBitmap((br.BaseStream as FileStream), imgBuffer, submapIndx, y);
                                    tr += shift;
                                }
                            }
                            else
                            {
                                int shift = AppendBitmapRaw((br.BaseStream as FileStream), imgBuffer, submapIndx, y);
                                tr += shift;
                            }
                        }
                    }
                }
            }

            bInfo.Width = width;
            bInfo.Height = height;

            bw.Close();
            br.Close();
        }

        private int InjectSmallerBiggerBitmap(FileStream fs, byte[] imgBuffer, int submapIndx, int LOD)
        {
            int tagIndx = Map.Tags.IndexOf(Identity);
            if (tagIndx == -1) return 0;

            BitReader br = new BitReader(fs, Endian.Little);
            BitWriter bw = new BitWriter(fs, Endian.Little);

            br.BaseStream.Position = Map.Tags.TagsList[tagIndx].MetaOffset + 68;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            br.BaseStream.Position = tr + (submapIndx * 116) + 28 + (LOD * 4);

            int rawOffset = br.ReadInt32();
            br.BaseStream.Position += 20;
            int rawSize = br.ReadInt32();
            int rawPadding = ((512 - (rawSize % 512)) % 512);

            br.BaseStream.Position = rawOffset + rawSize + rawPadding;
            byte[] mapEndHalf = br.ReadBytes((int)(br.BaseStream.Length - br.BaseStream.Position));

            bw.BaseStream.Position = rawOffset;
            bw.Write(imgBuffer);
            int shift = imgBuffer.Length;

            int newPadding = 512 - (imgBuffer.Length % 512);
            if (newPadding != 512)
            {
                bw.Write(new byte[newPadding]);
                shift += newPadding;
            }
            bw.Write(mapEndHalf);

            shift -= (rawSize + rawPadding);

            //--[Header]
            bw.BaseStream.Position = 16;
            bw.Write(Map.Header.IndexOffset += shift);

            //--[Shift]
            Map.Index.ShiftValuesInApp(shift);
            for (int x = 0; x < Map.Tags.TagsList.Length; x++)
            {
                if (Map.Tags.TagsList[x].Type != "sbsp" && Map.Tags.TagsList[x].Type != "ltmp")
                    Map.Tags.TagsList[x].MetaOffset += shift;
            }
            Map.Misc.ShiftBitmapRawPointers(fs, shift, rawOffset + 1);

            //--[RawSize]
            bw.BaseStream.Position = tr + (submapIndx * 116) + 28 + (LOD * 4) + 24 + shift;
            bw.Write(imgBuffer.Length);

            //--[End]
            bw.BaseStream.Position =
                Map.Tags.TagsList[Map.Tags.TagsList.Length - 1].MetaOffset +
                Map.Tags.TagsList[Map.Tags.TagsList.Length - 1].MetaSize;

            newPadding = 512 - ((int)bw.BaseStream.Position % 512);
            newPadding += (newPadding == 512 ? 512 : 1024);

            int eof = (int)bw.BaseStream.Position + newPadding;
            byte cd = 205;
            while (bw.BaseStream.Position < eof) bw.Write(cd);
            bw.BaseStream.SetLength(eof);

            //--[Header]
            bw.BaseStream.Position = 8;
            bw.Write(eof);

            return shift;
        }

        #region DDS Header

        /// <summary>
        /// The dwFlags member of the modified DDSURFACEDESC2 structure can be set to one 
        /// or more of the following values.
        /// </summary>
        enum DDSEnum : int
        {
            DDSD_CAPS = 0x1,
            DDSD_HEIGHT = 0x2,
            DDSD_WIDTH = 0x4,
            DDSD_PITCH = 0x8,
            DDSD_PIXELFORMAT = 0x1000,
            DDSD_MIPMAPCOUNT = 0x20000,
            DDSD_LINEARSIZE = 0x80000,
            DDSD_DEPTH = 0x800000,

            DDPF_ALPHAPIXELS = 0x1,
            DDPF_FOURCC = 0x4,
            DDPF_RGB = 0x40,

            //'The dwCaps1 member of the DDSCAPS2 structure can be set to one or more of the following values.
            DDSCAPS_COMPLEX = 0x8,
            DDSCAPS_TEXTURE = 0x1000,
            DDSCAPS_MIPMAP = 0x400000,

            //The dwCaps2 member of the DDSCAPS2 structure can be set to one or more of the following values.
            DDSCAPS2_CUBEMAP = 0x200,
            DDSCAPS2_CUBEMAP_POSITIVEX = 0x400,
            DDSCAPS2_CUBEMAP_NEGATIVEX = 0x800,
            DDSCAPS2_CUBEMAP_POSITIVEY = 0x1000,
            DDSCAPS2_CUBEMAP_NEGATIVEY = 0x2000,
            DDSCAPS2_CUBEMAP_POSITIVEZ = 0x4000,
            DDSCAPS2_CUBEMAP_NEGATIVEZ = 0x8000,
            DDSCAPS2_VOLUME = 0x200000
        }

        public struct DDS_Header
        {
            public string Magic;
            public DDS_SurfaceDesc dds_SurfaceDesc;

            public void ReadStruct(BitReader br)
            {
                Magic = new string(br.ReadChars(4));
                dds_SurfaceDesc.ReadStruct(br);
            }

            public void WriteStruct(BitWriter bw)
            {
                bw.Write(Magic.ToCharArray());
                dds_SurfaceDesc.WriteStruct(bw);
            }

            public void Generate(BitmapInfo bi)
            {
                Magic = "DDS ";
                dds_SurfaceDesc.Generate(bi);
            }

            public void FixBitmapInfo(ref BitmapInfo bi)
            {
                bi.Width = (ushort)dds_SurfaceDesc.width;
                bi.Height = (ushort)dds_SurfaceDesc.height;
                bi.Depth = (ushort)dds_SurfaceDesc.depth;
                //bi.Type = dds_SurfaceDesc.
                //bi.bitmFormat = dds_SurfaceDesc.dds_PixelFormat.
                //bi.Flags = dds_SurfaceDesc.
                //bi.RegPointX = dds_SurfaceDesc.
                //bi.RegPointY = dds_SurfaceDesc.
                bi.MipMapCount = (ushort)dds_SurfaceDesc.MipMapCount;
                //bi.PixelOffset = dds_SurfaceDesc.
            }
        }

        public struct DDS_SurfaceDesc
        {
            public int size_of_structure;
            public int flags;
            public int height;
            public int width;
            public int PitchOrLinearSize;
            public int depth; //Only used for volume textures.
            public int MipMapCount;
            public int[] Reserved1;
            public DDS_PixelFormat dds_PixelFormat;
            public DDS_Caps dds_Caps;
            public int Reserved2;

            public void ReadStruct(BitReader br)
            {
                size_of_structure = br.ReadInt32();
                flags = br.ReadInt32();
                height = br.ReadInt32();
                width = br.ReadInt32();
                PitchOrLinearSize = br.ReadInt32();
                depth = br.ReadInt32();
                MipMapCount = br.ReadInt32();

                Reserved1 = new int[11]; //11 ints long
                for (int x = 0; x < 11; x++)
                    Reserved1[x] = br.ReadInt32();

                dds_PixelFormat.ReadStruct(br);
                dds_Caps.ReadStruct(br);

                Reserved2 = br.ReadInt32();
            }

            public void WriteStruct(BitWriter bw)
            {
                bw.Write(size_of_structure);
                bw.Write(flags);
                bw.Write(height);
                bw.Write(width);
                bw.Write(PitchOrLinearSize);
                bw.Write(depth);
                bw.Write(MipMapCount);

                for (int x = 0; x < 11; x++)
                    bw.Write(Reserved1[x]);

                dds_PixelFormat.WriteStruct(bw);
                dds_Caps.WriteStruct(bw);

                bw.Write(Reserved2);
            }

            /// <summary>Generate the DDSURFACEDESC2 chunk of the header.</summary>
            public void Generate(BitmapInfo bi)
            {
                Reserved1 = new int[11];

                //'Size of structure. This member must be set to 124.
                size_of_structure = 124;

                //Flags to indicate valid fields. Always include DDSD_CAPS, DDSD_PIXELFORMAT,
                //DDSD_WIDTH, DDSD_HEIGHT and either DDSD_PITCH or DDSD_LINEARSIZE.
                flags = flags + (int)DDSEnum.DDSD_CAPS;
                flags = flags + (int)DDSEnum.DDSD_PIXELFORMAT;
                flags = flags + (int)DDSEnum.DDSD_WIDTH;
                flags = flags + (int)DDSEnum.DDSD_HEIGHT;

                switch (bi.bitmFormat)
                {
                    case BitmapFormat.DXT1:
                    case BitmapFormat.DXT2_3:
                    case BitmapFormat.DXT4_5:
                        flags = flags + (int)DDSEnum.DDSD_LINEARSIZE;
                        break;

                    default:
                        flags = flags + (int)DDSEnum.DDSD_PITCH;
                        break;
                }

                //Height of the main image in pixels
                height = bi.Height;

                //Width of the main image in pixels
                width = bi.Width;

                //For uncompressed formats, this is the number of bytes per scan line (DWORD aligned) for the main
                //image. dwFlags should include DDSD_PITCH in this case. For compressed formats, this is the (total)
                //number of bytes for the main image. dwFlags should be include DDSD_LINEARSIZE in this case.
                int RGBBitCount = 0;
                switch (bi.bitmFormat)
                {
                    case BitmapFormat.R5G6B5:
                    case BitmapFormat.A1R5G5B5:
                    case BitmapFormat.A4R4G4B4:
                        RGBBitCount = 16;
                        break;

                    case BitmapFormat.P8:
                    case BitmapFormat.X8R8G8B8:
                    case BitmapFormat.A8R8G8B8:
                        RGBBitCount = 32;
                        break;
                }

                PitchOrLinearSize = bi.Width * (RGBBitCount / 8);

                //For volume textures, this is the depth of the volume.
                //dwFlags should include DDSD_DEPTH in this case.
                depth = 0; // There are no volume textures (that I know of) in Halo.

                //For items with mipmap levels, this is the total number of levels in the mipmap
                //chain of the main image. dwFlags should include DDSD_MIPMAPCOUNT in this case
                MipMapCount = bi.MipMapCount;
                if (MipMapCount > 0) flags = flags + (int)DDSEnum.DDSD_MIPMAPCOUNT;

                //A 32-byte value that specifies the pixel format structure.
                dds_PixelFormat.Generate(bi);

                //A 16-byte value that specifies the capabilities structure.
                dds_Caps.Generate(bi);
            }
        }

        public struct DDS_PixelFormat
        {
            public int size; //32
            public int Flags; //4
            public string FourCC; //DXT1, DXT2, etc..
            public int RGBBitCount;
            public int RBitMask;
            public int GBitMask;
            public int BBitMask;
            public int RGBAlphaBitMask;

            public void ReadStruct(BitReader br)
            {
                size = br.ReadInt32(); //32
                Flags = br.ReadInt32(); //4
                FourCC = new string(br.ReadChars(4)); //DXT1, DXT2, etc..
                RGBBitCount = br.ReadInt32();
                RBitMask = br.ReadInt32();
                GBitMask = br.ReadInt32();
                BBitMask = br.ReadInt32();
                RGBAlphaBitMask = br.ReadInt32();
            }

            public void WriteStruct(BitWriter bw)
            {
                bw.Write(size);
                bw.Write(Flags);
                bw.Write(FourCC.ToCharArray());

                bw.Write(RGBBitCount);
                bw.Write(RBitMask);
                bw.Write(GBitMask);
                bw.Write(BBitMask);
                bw.Write(RGBAlphaBitMask);
            }

            /// <summary>Generate the DDPIXELFORMAT chunk of the header.</summary>
            public void Generate(BitmapInfo bi)
            {
                //Size of structure. This member must be set to 32. 
                size = 32;

                //Flags to indicate valid fields. Uncompressed formats will usually use DDPF_RGB to indicate
                //an RGB format, while compressed formats will use DDPF_FOURCC with a four-character code. 
                //This is accomplished in the below structure.

                //This is the four-character code for compressed formats. dwFlags should include DDPF_FOURCC in
                //this case. For DXTn compression, this is set to "DXT1", "DXT2", "DXT3", "DXT4", or "DXT5". 
                Flags = 0;
                switch (bi.bitmFormat)
                {
                    case BitmapFormat.DXT1:
                        FourCC = "DXT1";
                        Flags = Flags + (int)DDSEnum.DDPF_FOURCC;
                        break;
                    case BitmapFormat.DXT2_3:
                        FourCC = "DXT3";
                        Flags = Flags + (int)DDSEnum.DDPF_FOURCC;
                        break;
                    case BitmapFormat.DXT4_5:
                        FourCC = "DXT4";
                        Flags = Flags + (int)DDSEnum.DDPF_FOURCC;
                        break;
                    default:
                        FourCC = "\0\0\0\0";
                        Flags = Flags + (int)DDSEnum.DDPF_RGB;
                        break;
                }

                //For RGB formats, this is the total number of bits in the format. dwFlags should include DDPF_RGB
                //in this case. This value is usually 16, 24, or 32. For A8R8G8B8, this value would be 32. 
                switch (bi.bitmFormat)
                {
                    case BitmapFormat.R5G6B5:
                        RGBBitCount = 16;
                        break;

                    case BitmapFormat.A1R5G5B5:
                    case BitmapFormat.A4R4G4B4:
                        RGBBitCount = 16;
                        Flags = Flags + (int)DDSEnum.DDPF_ALPHAPIXELS;
                        break;

                    case BitmapFormat.X8R8G8B8:
                        //Flags = Flags + (int)DDSEnum.DDPF_ALPHAPIXELS;
                        RGBBitCount = 32;
                        break;

                    case BitmapFormat.P8:
                    case BitmapFormat.A8R8G8B8:
                        Flags = Flags + (int)DDSEnum.DDPF_ALPHAPIXELS;
                        RGBBitCount = 32;
                        break;

                    default:
                        RGBBitCount = 0;
                        break;
                }

                //For RGB formats, this contains the masks for the red, green, and blue channels. For A8R8G8B8, these
                //values would be 0x00ff0000, 0x0000ff00, and 0x000000ff respectively. 
                switch (bi.bitmFormat)
                {
                    case BitmapFormat.R5G6B5:
                        RBitMask = 0xF800;
                        GBitMask = 0x7E0;
                        BBitMask = 0x1F;
                        RGBAlphaBitMask = 0x0;
                        break;

                    case BitmapFormat.A1R5G5B5:
                        RBitMask = 0x7C00;
                        GBitMask = 0x3E0;
                        BBitMask = 0x1F;
                        RGBAlphaBitMask = 0x8000;
                        break;

                    case BitmapFormat.A4R4G4B4:
                        RBitMask = 0xF00;
                        GBitMask = 0xF0;
                        BBitMask = 0xF;
                        RGBAlphaBitMask = 0xF000;
                        break;

                    case BitmapFormat.X8R8G8B8:
                        RBitMask = 0xFF0000;
                        GBitMask = 0xFF00;
                        BBitMask = 0xFF;
                        RGBAlphaBitMask = 0x0;
                        break;

                    case BitmapFormat.P8:
                    case BitmapFormat.A8R8G8B8:
                        RBitMask = 0xFF0000;
                        GBitMask = 0xFF00;
                        BBitMask = 0xFF;
                        RGBAlphaBitMask = int.Parse("FF000000", System.Globalization.NumberStyles.HexNumber);
                        break;

                    default:
                        RBitMask = 0;
                        GBitMask = 0;
                        BBitMask = 0;
                        RGBAlphaBitMask = 0x0;
                        break;
                }

                //For RGB formats, this contains the mask for the alpha channel, if any. dwFlags should include 
                //DDPF_ALPHAPIXELS in this case. For A8R8G8B8, this value would be 0xff000000. 
                //^^ This was set in the previous block of code.
            }
        }

        public struct DDS_Caps
        {
            public int caps1;
            public int caps2;
            public int[] Reserved;

            public void ReadStruct(BitReader br)
            {
                caps1 = br.ReadInt32();
                caps2 = br.ReadInt32();
                Reserved = new int[2];
                Reserved[0] = br.ReadInt32();
                Reserved[1] = br.ReadInt32();
            }

            public void WriteStruct(BitWriter bw)
            {
                bw.Write(caps1);
                bw.Write(caps2);
                bw.Write(0);//Reserved[0]);
                bw.Write(0); //Reserved[1]);
            }

            /// <summary>Generate the DDSCAPS2 chunk of the header.</summary>
            public void Generate(BitmapInfo bi)
            {
                //DDS files should always include DDSCAPS_TEXTURE. If the file contains mipmaps, DDSCAPS_MIPMAP
                //should be set. For any DDS file with more than one main surface,such as a mipmaps, cubic
                //environment Map, or volume texture, DDSCAPS_COMPLEX should also be set. 
                caps1 = caps1 + (int)DDSEnum.DDSCAPS_TEXTURE;
                if (bi.MipMapCount > 0) caps1 = caps1 + (int)DDSEnum.DDSCAPS_MIPMAP;
                caps1 = caps1 + (int)DDSEnum.DDSCAPS_COMPLEX;

                //For cubic environment maps, DDSCAPS2_CUBEMAP should be included as well as one or more faces of
                //the Map (DDSCAPS2_CUBEMAP_POSITIVEX, DDSCAPS2_CUBEMAP_NEGATIVEX, DDSCAPS2_CUBEMAP_POSITIVEY,
                // DDSCAPS2_CUBEMAP_NEGATIVEY, DDSCAPS2_CUBEMAP_POSITIVEZ, DDSCAPS2_CUBEMAP_NEGATIVEZ). For volume
                //textures, DDSCAPS2_VOLUME should be included. 
                //******************************************************************************
                //This is where I stopped - this code needs to be added for cubemaps.
                //******************************************************************************
                caps2 = 0;
            }
        }

        #endregion

        #endregion

        #region Decoders

        private static byte[] DecodeDXT1(int height, int width, byte[] SourceData)
        {
            RGBA_IntColor[] Color = new RGBA_IntColor[5];

            RGBA_IntColor CColor;
            RGBA_IntColor zeroColor;

            int CData;
            int c1;
            int c2;
            int dptr = 0;

            bool trans;
            byte[] DestData = new byte[(width * height) * 4];

            int ChunksPerHLine = width / 4;
            if (ChunksPerHLine == 0) ChunksPerHLine = 1;

            for (int i = 0; i < (width * height); i += 16)
            {
                c1 = (SourceData[dptr + 1] << 8) | (SourceData[dptr]);
                c2 = (SourceData[dptr + 3] << 8) | (SourceData[dptr + 2]);

                trans = (!(c1 > c2));

                Color[0] = ShortToRGBA(c1);
                Color[1] = ShortToRGBA(c2);

                if (!trans)
                {
                    Color[2] = GradientColors(Color[0], Color[1]);
                    Color[3] = GradientColors(Color[1], Color[0]);
                }
                else
                {
                    zeroColor = Color[0];
                    Color[2] = GradientColorsHalf(Color[0], Color[1]);
                    Color[3] = zeroColor;
                }

                CData = (SourceData[dptr + 4] << 0) | (SourceData[dptr + 5] << 8) |
                    (SourceData[dptr + 6] << 16) | (SourceData[dptr + 7] << 24);

                int ChunkNum = i / 16;
                long XPos = ChunkNum % ChunksPerHLine;
                long YPos = (ChunkNum - XPos) / ChunksPerHLine;

                long tmp1, tmp2;

                int sizeh = height < 4 ? height : 4;
                int sizew = width < 4 ? width : 4;

                int x, y;
                for (x = 0; x < sizeh; x++)
                {
                    for (y = 0; y < sizew; y++)
                    {
                        CColor = Color[CData & 3];
                        CData >>= 2;
                        tmp1 = ((YPos * 4 + x) * width + XPos * 4 + y) * 4;
                        tmp2 = RGBAToInt(CColor);
                        DestData[tmp1] = Convert.ToByte(CColor.B);
                        DestData[tmp1 + 1] = Convert.ToByte(CColor.G);
                        DestData[tmp1 + 2] = Convert.ToByte(CColor.R);
                        DestData[tmp1 + 3] = Convert.ToByte(CColor.A);
                    }
                }
                dptr += 8;
            }
            return DestData;
        }

        private static byte[] DecodeDXT23(int height, int width, byte[] SourceData)
        {
            RGBA_IntColor[] Color = new RGBA_IntColor[5];

            RGBA_IntColor CColor;
            RGBA_IntColor c1, c2, c3, c4;
            int CData;
            byte[] DestData = new byte[(width * height) * 4];

            int ChunksPerHLine = width / 4;
            if (ChunksPerHLine == 0) ChunksPerHLine = 1;

            for (int i = 0; i < (width * height); i += 16)
            {
                c1 = ShortToRGBA((SourceData[i + 8]) | (SourceData[i + 9] << 8));
                c2 = ShortToRGBA((SourceData[i + 10]) | (SourceData[i + 11] << 8));
                c3 = GradientColors(Color[0], Color[1]);
                c4 = GradientColors(Color[1], Color[0]);
                Color[0] = c1;
                Color[1] = c2;
                Color[2] = c3;
                Color[3] = c4;

                CData = (SourceData[i + 12] << 0) | (SourceData[i + 13] << 8) | (SourceData[i + 14] << 16) | (SourceData[i + 15] << 24);

                int ChunkNum = i / 16;
                long XPos = ChunkNum % ChunksPerHLine;
                long YPos = (ChunkNum - XPos) / ChunksPerHLine;

                long ttmp;

                int alpha;
                int sizeh = height < 4 ? height : 4;
                int sizew = width < 4 ? width : 4;
                int x, y;

                for (x = 0; x < sizeh; x++)
                {
                    alpha = SourceData[i + (2 * x)] | (SourceData[i + (2 * x) + 1]) << 8;
                    for (y = 0; y < sizew; y++)
                    {
                        CColor = Color[CData & 3];
                        CData >>= 2;
                        CColor.A = (alpha & 15) * 16;
                        alpha >>= 4;
                        ttmp = ((YPos * 4 + x) * width + XPos * 4 + y) * 4;

                        DestData[ttmp] = (byte)CColor.B;
                        DestData[ttmp + 1] = (byte)CColor.G;
                        DestData[ttmp + 2] = (byte)CColor.R;
                        DestData[ttmp + 3] = (byte)CColor.A;
                    }
                }
            }
            return DestData;
        }

        private static byte[] DecodeDXT45(int height, int width, byte[] SourceData)
        {
            RGBA_IntColor[] Color = new RGBA_IntColor[4];
            RGBA_IntColor CColor;

            int CData;
            byte[] DestData = new byte[(width * height) * 4];

            int ChunksPerHLine = width / 4;
            if (ChunksPerHLine == 0) ChunksPerHLine = 1;

            for (int i = 0; i < (width * height); i += 16)
            {
                Color[0] = ShortToRGBA(SourceData[i + 8] | (SourceData[i + 9] << 8));
                Color[1] = ShortToRGBA(SourceData[i + 10] | (SourceData[i + 11] << 8));
                Color[2] = GradientColors(Color[0], Color[1]);
                Color[3] = GradientColors(Color[1], Color[0]);

                CData = (SourceData[i + 12] << 0) | (SourceData[i + 13] << 8) | (SourceData[i + 14] << 16) | (SourceData[i + 15] << 24);

                byte[] Alpha = new byte[8];
                Alpha[0] = SourceData[i];
                Alpha[1] = SourceData[i + 1];

                //Do the alphas
                if (Alpha[0] > Alpha[1])
                {
                    // 8-alpha block:  derive the other six alphas.
                    // Bit code 000 = alpha_0, 001 = alpha_1, others are interpolated.

                    Alpha[2] = (byte)((6 * Alpha[0] + 1 * Alpha[1] + 3) / 7); // bit code 010
                    Alpha[3] = (byte)((5 * Alpha[0] + 2 * Alpha[1] + 3) / 7); // bit code 011
                    Alpha[4] = (byte)((4 * Alpha[0] + 3 * Alpha[1] + 3) / 7); // bit code 100
                    Alpha[5] = (byte)((3 * Alpha[0] + 4 * Alpha[1] + 3) / 7); // bit code 101
                    Alpha[6] = (byte)((2 * Alpha[0] + 5 * Alpha[1] + 3) / 7); // bit code 110
                    Alpha[7] = (byte)((1 * Alpha[0] + 6 * Alpha[1] + 3) / 7); // bit code 111
                }
                else
                {
                    // 6-alpha block.
                    // Bit code 000 = alpha_0, 001 = alpha_1, others are interpolated.
                    Alpha[2] = (byte)((4 * Alpha[0] + 1 * Alpha[1] + 2) / 5); // Bit code 010
                    Alpha[3] = (byte)((3 * Alpha[0] + 2 * Alpha[1] + 2) / 5); // Bit code 011
                    Alpha[4] = (byte)((2 * Alpha[0] + 3 * Alpha[1] + 2) / 5); // Bit code 100
                    Alpha[5] = (byte)((1 * Alpha[0] + 4 * Alpha[1] + 2) / 5); // Bit code 101
                    Alpha[6] = 0;            // Bit code 110
                    Alpha[7] = 255;          // Bit code 111
                }

                // Byte	Alpha
                // 0	Alpha_0
                // 1	Alpha_1 
                // 2	(0)(2) (2 LSBs), (0)(1), (0)(0)
                // 3	(1)(1) (1 LSB), (1)(0), (0)(3), (0)(2) (1 MSB)
                // 4	(1)(3), (1)(2), (1)(1) (2 MSBs)
                // 5	(2)(2) (2 LSBs), (2)(1), (2)(0)
                // 6	(3)(1) (1 LSB), (3)(0), (2)(3), (2)(2) (1 MSB)
                // 7	(3)(3), (3)(2), (3)(1) (2 MSBs)
                // (0

                // Read an int and a short
                int tmpword = SourceData[i + 2] | (SourceData[i + 3] << 8);
                long tmpdword = SourceData[i + 4] | (SourceData[i + 5] << 8) | (SourceData[i + 6] << 16) | (SourceData[i + 7] << 24);

                long alphaDat = tmpword | (tmpdword << 16);

                int ChunkNum = i / 16;
                long XPos = ChunkNum % ChunksPerHLine;
                long YPos = (ChunkNum - XPos) / ChunksPerHLine;
                long ttmp;
                int sizeh = height < 4 ? height : 4;
                int sizew = width < 4 ? width : 4;
                int x, y;
                for (x = 0; x < sizeh; x++)
                {
                    for (y = 0; y < sizew; y++)
                    {
                        CColor = Color[CData & 3];
                        CData >>= 2;
                        CColor.A = Alpha[alphaDat & 7];
                        alphaDat >>= 3;
                        ttmp = ((YPos * 4 + x) * width + XPos * 4 + y) * 4;
                        DestData[ttmp] = (byte)CColor.B;
                        DestData[ttmp + 1] = (byte)CColor.G;
                        DestData[ttmp + 2] = (byte)CColor.R;
                        DestData[ttmp + 3] = (byte)CColor.A;
                    }
                }
            }
            return DestData;
        }

        public static Bitmap DecodeForBitmap(Map map, byte[] imgBuffer, BitmapInfo bitmapInfo, int paletteIndx, int sbspIndx)
        {
            int stride = bitmapInfo.Width;
            PixelFormat pixFormat; // = PixelFormat.Undefined;

            switch (bitmapInfo.bitmFormat)
            {
                // 32 bit
                case BitmapFormat.DXT1:
                    {
                        imgBuffer = DecodeDXT1(bitmapInfo.Height, bitmapInfo.Width, imgBuffer);
                        if (imgBuffer == null) return null;

                        stride *= 4;
                        pixFormat = PixelFormat.Format32bppArgb;
                        break;
                    }
                case BitmapFormat.DXT2_3:
                    {
                        imgBuffer = DecodeDXT23(bitmapInfo.Height, bitmapInfo.Width, imgBuffer);
                        if (imgBuffer == null) return null;

                        stride *= 4;
                        pixFormat = PixelFormat.Format32bppArgb;
                        break;
                    }
                case BitmapFormat.DXT4_5:
                    {
                        imgBuffer = DecodeDXT45(bitmapInfo.Height, bitmapInfo.Width, imgBuffer);
                        if (imgBuffer == null) return null;

                        stride *= 4;
                        pixFormat = PixelFormat.Format32bppArgb;
                        break;
                    }
                case BitmapFormat.A8R8G8B8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 32, true);

                        if (imgBuffer == null) return null;

                        stride *= 4;
                        pixFormat = PixelFormat.Format32bppArgb;
                        break;
                    }
                case BitmapFormat.X8R8G8B8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.Width, bitmapInfo.Height, -1, 32, true);

                        if (imgBuffer == null) return null;

                        stride *= 4;
                        pixFormat = PixelFormat.Format32bppArgb;
                        break;
                    }
                // 16 bit
                case BitmapFormat.A4R4G4B4:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, -1, 16, true);

                        if (imgBuffer == null) return null;

                        stride *= 2;
                        pixFormat = PixelFormat.Format16bppRgb565;
                        break;
                    }
                case BitmapFormat.Unknown:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            0, bitmapInfo.Width, bitmapInfo.Height, -1, 16, true);

                        if (imgBuffer == null) return null;

                        stride *= 2;
                        pixFormat = PixelFormat.Format32bppArgb;

                        int tmpHalf = imgBuffer.Length / 2;
                        byte[] tmpBuffer = new byte[tmpHalf * 4];
                        for (int x = 0; x < tmpHalf; x++)
                        {
                            int r = x * 2;
                            tmpBuffer[r * 2 + 0] = imgBuffer[r + 1];
                            tmpBuffer[r * 2 + 1] = imgBuffer[r + 1];
                            tmpBuffer[r * 2 + 2] = imgBuffer[r + 1];
                            tmpBuffer[r * 2 + 3] = imgBuffer[r + 0];
                        }
                        imgBuffer = tmpBuffer;
                        break;
                    }
                case BitmapFormat.A1R5G5B5:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, -1, 16, true);

                        if (imgBuffer == null) return null;

                        stride *= 2;
                        pixFormat = PixelFormat.Format16bppRgb555;
                        break;
                    }
                case BitmapFormat.R5G6B5:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, -1, 16, true);

                        if (imgBuffer == null) return null;

                        stride *= 2;
                        pixFormat = PixelFormat.Format16bppRgb565;
                        break;
                    }
                case BitmapFormat.A8Y8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 16, true);

                        if (imgBuffer == null) return null;

                        stride *= 4;
                        pixFormat = PixelFormat.Format32bppArgb;

                        int tmpHalf = imgBuffer.Length / 2;
                        byte[] tmpBuffer = new byte[tmpHalf * 4];
                        for (int x = 0; x < tmpHalf; x++)
                        {
                            int r = x * 2;
                            tmpBuffer[r + 0] = imgBuffer[r + 1];
                            tmpBuffer[r + 1] = imgBuffer[r + 1];
                            tmpBuffer[r + 2] = imgBuffer[r + 1];
                            tmpBuffer[r + 3] = imgBuffer[r + 0];
                        }
                        imgBuffer = tmpBuffer;
                        break;
                    }
                // 8 bit
                case BitmapFormat.P8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 8, true);

                        if (imgBuffer == null) return null;

                        stride *= 4;
                        pixFormat = PixelFormat.Format32bppArgb;

                        if (!H2PaletteLoaded) LoadHalo2Palette();

                        byte[] tmpBuffer = new byte[imgBuffer.Length * 4];
                        for (int x = 0; x < imgBuffer.Length; x++)
                        {
                            int r = x * 4;
                            tmpBuffer[r] = H2Palette[imgBuffer[x]].R;
                            tmpBuffer[r + 1] = H2Palette[imgBuffer[x]].G;
                            tmpBuffer[r + 2] = H2Palette[imgBuffer[x]].B;
                            tmpBuffer[r + 3] = H2Palette[imgBuffer[x]].A;
                        }
                        imgBuffer = tmpBuffer;
                        break;
                    }
                case BitmapFormat.Lightmap:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 8, true);

                        if (imgBuffer == null || paletteIndx == 255) return null;

                        stride *= 4;
                        pixFormat = PixelFormat.Format32bppArgb;

                        byte[] tmpBuffer = new byte[imgBuffer.Length * 4];
                        for (int x = 0; x < imgBuffer.Length; x++)
                        {
                            int r = x * 4;
                            tmpBuffer[r + 0] = map.SBSP.BSPDeclarations[sbspIndx].Lightmap.Palettes[paletteIndx][imgBuffer[x]].R;
                            tmpBuffer[r + 1] = map.SBSP.BSPDeclarations[sbspIndx].Lightmap.Palettes[paletteIndx][imgBuffer[x]].G;
                            tmpBuffer[r + 2] = map.SBSP.BSPDeclarations[sbspIndx].Lightmap.Palettes[paletteIndx][imgBuffer[x]].B;
                            tmpBuffer[r + 3] = map.SBSP.BSPDeclarations[sbspIndx].Lightmap.Palettes[paletteIndx][imgBuffer[x]].A;
                        }
                        imgBuffer = tmpBuffer;
                        break;
                    }
                case BitmapFormat.Y8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 8, true);

                        if (imgBuffer == null) return null;

                        stride *= 4;
                        pixFormat = PixelFormat.Format32bppArgb;

                        byte[] tmpBuffer = new byte[imgBuffer.Length * 4];
                        for (int x = 0; x < imgBuffer.Length; x++)
                        {
                            int r = x * 4;
                            tmpBuffer[r + 0] = imgBuffer[x];
                            tmpBuffer[r + 1] = imgBuffer[x];
                            tmpBuffer[r + 2] = imgBuffer[x];
                            tmpBuffer[r + 3] = imgBuffer[255];
                        }
                        imgBuffer = tmpBuffer;
                        break;
                    }
                default:
                    {
                        return null;
                    }
            }

            IntPtr ptr = new IntPtr();
            Marshal.FreeHGlobal(ptr);
            ptr = Marshal.AllocHGlobal(imgBuffer.Length);
            RtlMoveMemory(ptr, imgBuffer, imgBuffer.Length);

            return new Bitmap(bitmapInfo.Width, bitmapInfo.Height, stride, pixFormat, ptr);
        }

        public static byte[] DecodeForDDS(Map map, byte[] imgBuffer, BitmapInfo bitmapInfo, int paletteIndx, int sbspIndx)
        {
            switch (bitmapInfo.bitmFormat)
            {
                // 32 bit
                case BitmapFormat.A8R8G8B8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 32, true);
                        break;
                    }
                case BitmapFormat.X8R8G8B8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.Width, bitmapInfo.Height, -1, 32, true);
                        break;
                    }
                // 16 bit
                case BitmapFormat.A4R4G4B4:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, -1, 16, true);
                        break;
                    }
                case BitmapFormat.Unknown:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            0, bitmapInfo.Width, bitmapInfo.Height, -1, 16, true);

                        int tmpHalf = imgBuffer.Length / 2;
                        byte[] tmpBuffer = new byte[tmpHalf * 4];
                        for (int x = 0; x < tmpHalf; x++)
                        {
                            int r = x * 2;
                            tmpBuffer[r * 2 + 0] = imgBuffer[r + 1];
                            tmpBuffer[r * 2 + 1] = imgBuffer[r + 1];
                            tmpBuffer[r * 2 + 2] = imgBuffer[r + 1];
                            tmpBuffer[r * 2 + 3] = imgBuffer[r + 0];
                        }
                        imgBuffer = tmpBuffer;
                        break;
                    }
                case BitmapFormat.A1R5G5B5:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, -1, 16, true);
                        break;
                    }
                case BitmapFormat.R5G6B5:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, -1, 16, true);
                        break;
                    }
                case BitmapFormat.A8Y8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 16, true);

                        int tmpHalf = imgBuffer.Length / 2;
                        byte[] tmpBuffer = new byte[tmpHalf * 4];
                        for (int x = 0; x < tmpHalf; x++)
                        {
                            int r = x * 2;
                            tmpBuffer[r + 0] = imgBuffer[r + 1];
                            tmpBuffer[r + 1] = imgBuffer[r + 1];
                            tmpBuffer[r + 2] = imgBuffer[r + 1];
                            tmpBuffer[r + 3] = imgBuffer[r + 0];
                        }
                        imgBuffer = tmpBuffer;
                        break;
                    }
                // 8 bit
                case BitmapFormat.P8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 8, true);

                        if (!H2PaletteLoaded) LoadHalo2Palette();

                        byte[] tmpBuffer = new byte[imgBuffer.Length * 4];
                        for (int x = 0; x < imgBuffer.Length; x++)
                        {
                            int r = x * 4;
                            tmpBuffer[r] = H2Palette[imgBuffer[x]].R;
                            tmpBuffer[r + 1] = H2Palette[imgBuffer[x]].G;
                            tmpBuffer[r + 2] = H2Palette[imgBuffer[x]].B;
                            tmpBuffer[r + 3] = H2Palette[imgBuffer[x]].A;
                        }
                        imgBuffer = tmpBuffer;
                        break;
                    }
                case BitmapFormat.Lightmap:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 8, true);

                        int pIndx;
                        if (paletteIndx < 0)
                        {
                            pIndx = map.SBSP.BSPDeclarations[sbspIndx].Lightmap.
                                PermPaletteIndices[0 - (paletteIndx + 1)];
                        }
                        else
                        {
                            pIndx = map.SBSP.BSPDeclarations[sbspIndx].Lightmap.
                                PermPaletteIndices[paletteIndx];
                        }
                        if (pIndx == 255) return null;

                        byte[] tmpBuffer = new byte[imgBuffer.Length * 4];
                        for (int x = 0; x < tmpBuffer.Length; x++)
                        {
                            int r = x * 4;
                            tmpBuffer[r + 0] = map.SBSP.BSPDeclarations[sbspIndx].Lightmap.Palettes[pIndx][imgBuffer[x]].R;
                            tmpBuffer[r + 1] = map.SBSP.BSPDeclarations[sbspIndx].Lightmap.Palettes[pIndx][imgBuffer[x]].G;
                            tmpBuffer[r + 2] = map.SBSP.BSPDeclarations[sbspIndx].Lightmap.Palettes[pIndx][imgBuffer[x]].B;
                            tmpBuffer[r + 3] = map.SBSP.BSPDeclarations[sbspIndx].Lightmap.Palettes[pIndx][imgBuffer[x]].A;
                        }
                        imgBuffer = tmpBuffer;
                        break;
                    }
                case BitmapFormat.Y8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 8, true);

                        byte[] tmpBuffer = new byte[imgBuffer.Length * 4];
                        for (int x = 0; x < imgBuffer.Length; x++)
                        {
                            int r = x * 4;
                            tmpBuffer[r + 0] = imgBuffer[x];
                            tmpBuffer[r + 1] = imgBuffer[x];
                            tmpBuffer[r + 2] = imgBuffer[x];
                            tmpBuffer[r + 3] = imgBuffer[255];
                        }
                        imgBuffer = tmpBuffer;
                        break;
                    }
            }

            return imgBuffer;
        }

        #endregion

        #region Encoders

        public byte[] EncodeForMap(byte[] imgBuffer, BitmapInfo bitmapInfo, int paletteIndx, int sbspIndx)
        {
            switch (bitmapInfo.bitmFormat)
            {
                // 32 bit
                case BitmapFormat.A8R8G8B8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 32, false);
                        break;
                    }
                case BitmapFormat.X8R8G8B8:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 32, false);
                        break;
                    }
                // 16 bit
                case BitmapFormat.A4R4G4B4:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, -1, 16, false);
                        break;
                    }
                case BitmapFormat.Unknown:
                    {
                        int tmpHalf = imgBuffer.Length * 2;
                        byte[] tmpBuffer = new byte[tmpHalf / 4];
                        for (int x = 0; x < (imgBuffer.Length / 4); x++)
                        {
                            int r = x * 2;
                            tmpBuffer[r + 1] = imgBuffer[r * 2 + 0];
                            tmpBuffer[r + 1] = imgBuffer[r * 2 + 1];
                            tmpBuffer[r + 1] = imgBuffer[r * 2 + 2];
                            tmpBuffer[r + 0] = imgBuffer[r * 2 + 3];
                        }
                        imgBuffer = tmpBuffer;

                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            0, bitmapInfo.Width, bitmapInfo.Height, -1, 16, false);
                        break;
                    }
                case BitmapFormat.A1R5G5B5:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, -1, 16, false);
                        break;
                    }
                case BitmapFormat.R5G6B5:
                    {
                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, -1, 16, false);
                        break;
                    }
                case BitmapFormat.A8Y8:
                    {
                        int tmpHalf = imgBuffer.Length * 2;
                        byte[] tmpBuffer = new byte[tmpHalf / 4];
                        for (int x = 0; x < (imgBuffer.Length / 4); x++)
                        {
                            int r = x * 2;
                            tmpBuffer[r + 1] = imgBuffer[r + 0];
                            tmpBuffer[r + 1] = imgBuffer[r + 1];
                            tmpBuffer[r + 1] = imgBuffer[r + 2];
                            tmpBuffer[r + 0] = imgBuffer[r + 3];
                        }
                        imgBuffer = tmpBuffer;

                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 16, false);
                        break;
                    }
                // 8 bit
                case BitmapFormat.P8:
                    {
                        if (!H2PaletteLoaded) LoadHalo2Palette();
                        byte[] tmpBuffer = new byte[imgBuffer.Length / 4];

                        RGBA_ByteColor baColor = new RGBA_ByteColor();
                        for (int x = 0; x < tmpBuffer.Length; x++)
                        {
                            int r = x * 4;
                            baColor.R = imgBuffer[r];
                            baColor.G = imgBuffer[r + 1];
                            baColor.B = imgBuffer[r + 2];
                            baColor.A = imgBuffer[r + 3];

                            byte colorIndx = 0;
                            float storedDiff = 0;// 195075f;
                            RGBA_ByteColor palColor = new RGBA_ByteColor();
                            for (byte y = 0; y < 255; y++)
                            {
                                palColor.R = H2Palette[y].R;
                                palColor.G = H2Palette[y].G;
                                palColor.B = H2Palette[y].B;
                                palColor.A = H2Palette[y].A;

                                float diff =
                                    (((baColor.R - palColor.R) * (baColor.R - palColor.R)) +
                                    ((baColor.G - palColor.G) * (baColor.G - palColor.G))) +
                                    ((baColor.B - palColor.B) * (baColor.B - palColor.B));

                                if (diff <= storedDiff)
                                {
                                    colorIndx = y;
                                    storedDiff = diff;
                                }
                            }
                            tmpBuffer[x] = colorIndx;
                        }
                        imgBuffer = tmpBuffer;

                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 8, false);
                        break;
                    }
                case BitmapFormat.Y8:
                    {
                        byte[] tmpBuffer = new byte[imgBuffer.Length / 4];
                        for (int x = 0; x < tmpBuffer.Length; x++)
                        {
                            int r = x * 4;
                            tmpBuffer[x] = imgBuffer[r + 0];
                            tmpBuffer[x] = imgBuffer[r + 1];
                            tmpBuffer[x] = imgBuffer[r + 2];
                            tmpBuffer[255] = imgBuffer[r + 3];
                        }
                        imgBuffer = tmpBuffer;

                        if (bitmapInfo.Swizzled) imgBuffer = Swizzle(imgBuffer,
                            bitmapInfo.PixelOffset, bitmapInfo.Width, bitmapInfo.Height, bitmapInfo.Depth, 8, false);
                        break;
                    }
            }

            return imgBuffer;
        }

        #endregion

        #region Swizzler

        public static byte[] Swizzle(byte[] raw, int pixOffset, int width, int height, int depth, int bitCount, bool deswizzle)
        {
            bitCount /= 8;
            int a = 0;
            int b = 0;
            byte[] dataArray = new byte[raw.Length]; //width * height * bitCount;

            MaskSet masks = new MaskSet(width, height, depth);
            pixOffset = 0;

            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                {
                    if (deswizzle)
                    {
                        a = ((y * width) + x) * bitCount;
                        b = (Swizzle(x, y, -1, masks)) * bitCount;
                    }
                    else
                    {
                        b = ((y * width) + x) * bitCount;
                        a = (Swizzle(x, y, -1, masks)) * bitCount;
                    }

                    if (a < dataArray.Length && b < raw.Length)
                    {
                        for (int i = pixOffset; i < bitCount + pixOffset; i++)
                            dataArray[a + i] = raw[b + i];
                    }
                    else return null;
                }
            }

            //for(int u = 0; u < offset; u++)
            //data[u] = raw[u];
            //for(int v = offset + (height * width * depth * bitCount); v < data.Length; v++)
            //	data[v] = raw[v];

            return dataArray;
        }

        public static byte[] Swizzle(byte[] raw, int width, int height, int depth, int bitCount, bool deswizzle)
        {
            return Swizzle(raw, 0, width, height, depth, bitCount, deswizzle);
        }

        private static int Swizzle(int x, int y, int z, MaskSet masks)
        {
            return SwizzleAxis(x, masks.x) | SwizzleAxis(y, masks.y) | (z == -1 ? 0 : SwizzleAxis(z, masks.z));
        }

        private static int SwizzleAxis(int val, int mask)
        {
            int bit = 1;
            int result = 0;

            while (bit <= mask)
            {
                int tmp = mask & bit;

                if (tmp != 0) result |= (val & bit);
                else val <<= 1;

                bit <<= 1;
            }

            return result;
        }

        private class MaskSet
        {
            public int x = 0;
            public int y = 0;
            public int z = 0;

            public MaskSet(int w, int h, int d)
            {
                int bit = 1;
                int index = 1;

                while (bit < w || bit < h || bit < d)
                {
                    //if (bit == 0) break;
                    if (bit < w)
                    {
                        x |= index;
                        index <<= 1;
                    }
                    if (bit < h)
                    {
                        y |= index;
                        index <<= 1;
                    }
                    if (bit < d)
                    {
                        z |= index;
                        index <<= 1;
                    }
                    bit <<= 1;
                }
            }
        }

        #endregion

        #region Misc

        public enum BitmapFormat : int
        {
            A8 = 0,
            Y8 = 1,
            AY8 = 2,
            A8Y8 = 3,
            R5G6B5 = 6,
            A1R5G5B5 = 8,
            A4R4G4B4 = 9,
            X8R8G8B8 = 10,
            A8R8G8B8 = 11,
            DXT1 = 14,
            DXT2_3 = 15,
            DXT4_5 = 16,
            P8 = 17,
            Lightmap = 18,
            Unknown = 22,

            _2D = 0,
            _3D = 1,
            Cubemap = 2
        }

        public struct BitmapInfo
        {
            public long mapOffset;

            public byte[] TagType;
            public ushort Width;
            public ushort Height;
            public ushort Depth;
            public ushort Type;
            public BitmapFormat bitmFormat;
            public bool Swizzled;
            public ushort Flags;
            public ushort RegPointX;
            public ushort RegPointY;
            public ushort MipMapCount;
            public ushort PixelOffset;

            public BitmapInfo(BitReader br)
            {
                mapOffset = br.BaseStream.Position;
                TagType = br.ReadBytes(4);
                Width = br.ReadUInt16();
                Height = br.ReadUInt16();
                Depth = br.ReadUInt16();
                Type = br.ReadUInt16();
                bitmFormat = (BitmapFormat)br.ReadUInt16();
                Flags = br.ReadUInt16();
                Swizzled = ((Flags & 0x8) == 0x8);
                RegPointX = br.ReadUInt16();
                RegPointY = br.ReadUInt16();
                MipMapCount = br.ReadUInt16();
                PixelOffset = br.ReadUInt16();
            }

            public void WriteInfo(BitWriter bw)
            {
                bw.Write(TagType);
                bw.Write(Width);
                bw.Write(Height);
                bw.Write(Depth);
                bw.Write(Type);
                bw.Write((ushort)bitmFormat);
                bw.Write(Flags);
                bw.Write(RegPointX);
                bw.Write(RegPointY);
                bw.Write(MipMapCount);
                bw.Write(PixelOffset);
            }
        }

        public struct RGBA_IntColor
        {
            public int R, G, B, A;
        }

        public struct RGBA_ByteColor
        {
            public byte R, G, B, A;
        }

        private static RGBA_IntColor ShortToRGBA(int color)
        {
            RGBA_IntColor rcs;
            color = Convert.ToUInt16(color);
            rcs.R = (((color >> 11) & 31) * 255) / 31;
            rcs.G = (((color >> 5) & 63) * 255) / 63;
            rcs.B = (((color >> 0) & 31) * 255) / 31;
            rcs.A = 255;
            return rcs;
        }

        private static int RGBAToInt(RGBA_IntColor rcs)
        {
            return (rcs.A << 24) | (rcs.R << 16) | (rcs.G << 8) | rcs.B;
        }

        private static RGBA_IntColor GradientColors(RGBA_IntColor Col1, RGBA_IntColor Col2)
        {
            RGBA_IntColor ret;
            ret.R = ((Col1.R * 2 + Col2.R)) / 3;
            ret.G = ((Col1.G * 2 + Col2.G)) / 3;
            ret.B = ((Col1.B * 2 + Col2.B)) / 3;
            ret.A = 255;
            return ret;
        }

        private static RGBA_IntColor GradientColorsHalf(RGBA_IntColor Col1, RGBA_IntColor Col2)
        {
            RGBA_IntColor ret;
            ret.R = (Col1.R / 2 + Col2.R / 2);
            ret.G = (Col1.G / 2 + Col2.G / 2);
            ret.B = (Col1.B / 2 + Col2.B / 2);
            ret.A = 255;
            return ret;
        }

        private static void LoadHalo2Palette()
        {
            #region H2Palette

            byte[] H2PaletteBuffer = new byte[] {
                255, 126, 126, 255, 255, 127, 126, 255, 255, 128, 126, 255, 255, 129, 126, 255, 255, 126, 
                127, 255, 255, 127, 127, 255, 255, 128, 127, 255, 255, 129, 127, 255, 255, 126, 128, 255, 
                255, 127, 128, 255, 255, 128, 128, 255, 255, 129, 128, 255, 255, 126, 129, 255, 255, 127, 
                129, 255, 255, 128, 129, 255, 255, 129, 129, 255, 255, 130, 127, 255, 255, 127, 131, 255, 
                255, 127, 125, 255, 255, 131, 129, 255, 255, 124, 129, 255, 255, 130, 124, 255, 255, 129, 
                132, 255, 255, 124, 125, 255, 255, 133, 127, 255, 255, 125, 132, 255, 255, 128, 122, 255, 
                255, 132, 132, 255, 255, 122, 128, 255, 255, 133, 124, 255, 255, 127, 135, 255, 255, 124, 
                122, 255, 255, 136, 130, 255, 255, 121, 132, 255, 255, 131, 120, 255, 255, 132, 136, 255, 
                255, 119, 124, 255, 255, 137, 125, 255, 255, 123, 137, 255, 255, 125, 118, 255, 255, 137, 
                134, 255, 255, 117, 130, 255, 255, 135, 119, 255, 255, 129, 140, 255, 255, 119, 120, 255, 
                255, 141, 128, 255, 255, 119, 137, 255, 255, 129, 115, 255, 255, 136, 139, 255, 255, 114, 
                126, 255, 255, 140, 120, 255, 255, 124, 142, 255, 255, 121, 115, 255, 255, 142, 133, 255, 
                255, 113, 134, 255, 254, 135, 113, 255, 254, 133, 144, 255, 254, 113, 120, 255, 254, 145, 
                124, 255, 254, 118, 142, 255, 254, 126, 110, 255, 254, 142, 140, 255, 254, 109, 129, 255, 
                254, 142, 114, 255, 254, 127, 147, 255, 254, 115, 113, 255, 254, 148, 131, 255, 254, 111, 
                140, 255, 254, 133, 107, 255, 254, 139, 147, 255, 254, 107, 121, 255, 254, 148, 119, 255, 
                253, 119, 149, 255, 253, 120, 106, 255, 253, 149, 139, 255, 253, 105, 134, 255, 253, 141, 
                108, 255, 253, 132, 152, 255, 253, 108, 113, 255, 253, 153, 126, 255, 253, 111, 147, 255, 
                253, 128, 102, 255, 253, 146, 147, 255, 253, 101, 126, 255, 253, 150, 111, 255, 252, 123, 
                155, 255, 252, 113, 104, 255, 252, 155, 135, 255, 252, 103, 141, 255, 252, 138, 101, 255, 
                252, 139, 155, 255, 252, 101, 115, 255, 252, 157, 119, 255, 252, 113, 155, 255, 252, 121, 
                98, 255, 252, 154, 146, 255, 251, 96, 132, 255, 251, 149, 103, 255, 251, 129, 161, 255, 
                251, 105, 105, 255, 251, 161, 129, 255, 251, 102, 150, 255, 251, 132, 94, 255, 251, 148, 
                156, 255, 251, 94, 120, 255, 251, 159, 110, 255, 250, 117, 162, 255, 250, 113, 95, 255, 
                250, 162, 142, 255, 250, 93, 141, 255, 250, 145, 95, 255, 250, 138, 164, 255, 250, 96, 
                108, 255, 250, 166, 121, 255, 249, 104, 159, 255, 249, 125, 89, 255, 249, 157, 155, 255, 
                249, 88, 128, 255, 249, 158, 101, 255, 249, 124, 169, 255, 249, 103, 95, 255, 248, 169, 
                135, 255, 248, 92, 151, 255, 248, 139, 87, 255, 248, 148, 166, 255, 248, 87, 113, 255, 
                248, 168, 111, 255, 248, 109, 168, 255, 247, 115, 86, 255, 247, 167, 150, 255, 247, 84, 
                138, 255, 247, 154, 91, 255, 247, 134, 174, 255, 247, 92, 98, 255, 247, 175, 126, 255, 
                246, 94, 162, 255, 246, 130, 80, 255, 246, 159, 165, 255, 246, 80, 122, 255, 246, 168, 
                100, 255, 246, 117, 176, 255, 245, 103, 85, 255, 245, 176, 143, 255, 245, 82, 149, 255, 
                245, 148, 81, 255, 245, 146, 176, 255, 244, 82, 104, 255, 244, 178, 114, 255, 244, 100, 
                172, 255, 244, 119, 76, 255, 244, 170, 161, 255, 244, 74, 133, 255, 243, 165, 88, 255, 
                243, 128, 183, 255, 243, 91, 87, 255, 243, 183, 133, 255, 243, 84, 162, 255, 242, 138, 
                73, 255, 242, 158, 176, 255, 242, 73, 113, 255, 242, 179, 101, 255, 242, 108, 182, 255, 
                241, 106, 74, 255, 241, 181, 153, 255, 241, 72, 146, 255, 241, 158, 76, 255, 240, 141, 
                187, 255, 240, 79, 93, 255, 240, 188, 120, 255, 240, 89, 175, 255, 240, 125, 66, 255, 
                239, 172, 172, 255, 239, 66, 125, 255, 239, 176, 88, 255, 239, 120, 191, 255, 238, 92, 
                76, 255, 238, 191, 142, 255, 238, 72, 160, 255, 238, 148, 66, 255, 237, 156, 187, 255, 
                237, 67, 103, 255, 237, 190, 105, 255, 237, 97, 187, 255, 237, 111, 63, 255, 236, 185, 
                164, 255, 236, 61, 140, 255, 236, 170, 74, 255, 235, 134, 196, 255, 235, 77, 81, 255, 
                235, 197, 128, 255, 235, 77, 175, 255, 234, 134, 58, 255, 234, 171, 184, 255, 234, 58, 
                116, 255, 234, 188, 90, 255, 233, 109, 197, 255, 233, 95, 64, 255, 233, 196, 153, 255, 
                233, 61, 156, 255, 232, 159, 62, 255, 232, 150, 198, 255, 232, 64, 91, 255, 231, 201, 
                112, 255, 231, 85, 189, 255, 231, 118, 53, 255, 231, 186, 177, 255, 230, 52, 131, 255, 
                230, 182, 74, 255, 230, 125, 205, 255, 229, 78, 69, 255, 229, 205, 138, 255, 229, 64, 
                173, 255, 228, 145, 51, 255, 228, 167, 196, 255, 228, 52, 104, 255, 227, 200, 94, 255, 
                227, 97, 202, 255, 227, 101, 52, 255, 227, 200, 165, 255, 226, 49, 149, 255, 226, 172, 
                59, 255, 226, 142, 209, 255, 225, 63, 78, 255, 225, 211, 121, 255, 225, 72, 189, 255, 
                224, 128, 44, 255, 224, 185, 190, 255, 224, 44, 121, 255, 223, 195, 76, 255, 223, 113, 
                212, 255, 223, 82, 56, 255, 222, 211, 150, 255, 222, 51, 168, 255, 221, 158, 47, 255, 
                221, 161, 209, 255, 221, 49, 91, 255, 220, 212, 102, 255, 220, 84, 204, 255, 220, 109, 
                41, 255, 219, 201, 179, 255, 219, 39, 140, 255, 219, 186, 59, 255, 218, 132, 218, 255, 
                218, 64, 64, 255, 217, 219, 132, 255, 217, 58, 187, 255, 217, 140, 37, 255, 216, 181, 
                203, 255, 216, 38, 108, 255, 216, 208, 82, 255, 215, 100, 217, 255, 215, 89, 43, 255, 
                214, 215, 164, 255, 214, 39, 160, 255, 214, 172, 44, 255, 255, 128, 128, 0, };

            #endregion

            H2Palette = new RGBA_ByteColor[256];
            for (int x = 0; x < 256; x++)
            {
                int r = x * 4;
                H2Palette[x].R = H2PaletteBuffer[r];
                H2Palette[x].G = H2PaletteBuffer[r + 1];
                H2Palette[x].B = H2PaletteBuffer[r + 2];
                H2Palette[x].A = H2PaletteBuffer[r + 3];
            }

            H2PaletteLoaded = true;
        }

        #endregion

        #endregion
    }
}