using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections.Generic;
using System.IO;
using System.Extended;
using H2UtilityToolkit;
using H2UtilityToolkit.Misc;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Shader
    /// </summary>
    public class Shad
    {
        #region Instance Data

        #region Constants

        /// <summary>
        /// Size of the shad meta header.
        /// </summary>
        public const int HeaderSize = 84;

        public const string SEP_cubeBitmString = "Cube map";
        public const string SEP_detaBitmString = "Detail map";
        public const string SEP_bumpBitmString = "Bump map";
        public const string SEP_mainBitmString = "Main";
        public const string SEP_unknBitmString = "Unknown";

        #endregion

        private Map __map__;
        private int __identity__;
        private string __shaderName__;
        private AlphaType __alpha__;
        private BitmapUVWScale[] __bitmUVWScales__;
        private ShaderTextureInfo __mainBitm__;
        private ShaderTextureInfo __bumpBitm__;
        private ShaderTextureInfo __detailBitm__;
        private ShaderTextureInfo __cubeBitm__;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return __map__; } set { __map__ = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return __identity__; } set { __identity__ = value; } }
        /// <summary>
        /// ShaderName
        /// </summary>
        public string ShaderName { get { return __shaderName__; } set { __shaderName__ = value; } }
        /// <summary>
        /// Alpha
        /// </summary>
        public AlphaType Alpha { get { return __alpha__; } set { __alpha__ = value; } }
        /// <summary>
        /// BitmUVWScales
        /// </summary>
        public BitmapUVWScale[] BitmUVWScales { get { return __bitmUVWScales__; } set { __bitmUVWScales__ = value; } }
        /// <summary>
        /// MainBitm
        /// </summary>
        public ShaderTextureInfo MainBitm { get { return __mainBitm__; } set { __mainBitm__ = value; } }
        /// <summary>
        /// BumpBitm
        /// </summary>
        public ShaderTextureInfo BumpBitm { get { return __bumpBitm__; } set { __bumpBitm__ = value; } }
        /// <summary>
        /// DetailBitm
        /// </summary>
        public ShaderTextureInfo DetailBitm { get { return __detailBitm__; } set { __detailBitm__ = value; } }
        /// <summary>
        /// CubeBitm
        /// </summary>
        public ShaderTextureInfo CubeBitm { get { return __cubeBitm__; } set { __cubeBitm__ = value; } }

        #endregion

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Shad class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Shad(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
        }

        #region Specific Methods of shad

        public Shad(Map map, int identity, BitReader br, bool loadTextures)
        {
            this.Map = map;
            this.Identity = identity;
            this.Read(br, loadTextures);
        }

        public void Read(BitReader br, bool loadTextures)
        {
            // Get shad index
            int shadIndx = Map.Tags.IndexOf(__identity__);
            if (shadIndx == -1) return;

            // Get path
            ShaderName = Map.Tags.TagsList[shadIndx].Path;

            // Goto main reflexive
            br.BaseStream.Position = Map.Tags.TagsList[shadIndx].MetaOffset + 32;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            br.BaseStream.Position = tr;

            // Get stem
            int stemIndx = Map.Tags.IndexOf(br.ReadInt32());

            //if (stemIndx == -1) return;
            if (stemIndx != -1)
            {
                // Get alpha
                if (Map.Tags.TagsList[stemIndx].Path.Contains("alphatest"))
                {
                    Alpha = AlphaType.AlphaTest;
                }
                else if (Map.Tags.TagsList[stemIndx].Path.Contains("alpha") || Map.Tags.TagsList[stemIndx].Path.Contains("water"))
                {
                    Alpha = AlphaType.AlphaBlend;
                }
            }

            // Bitmap inner-reflexive
            cc = br.ReadInt32();
            tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            // Loop bitmaps
            for (int x = 0; x < cc; x++)
            {
                br.BaseStream.Position = tr + (x * 12);

                // Get bitmap index
                int bitmIndx = Map.Tags.IndexOf(br.ReadInt32());
                if (bitmIndx == -1) continue;

                // Goto address0 in bitm tag
                br.BaseStream.Position = Map.Tags.TagsList[bitmIndx].MetaOffset;

                // Enum16's
                int type = br.ReadInt16();
                int format = br.ReadInt16();
                int usage = br.ReadInt16();

                if (type == 2) // (\reflection_maps\) OR (_cube_map)
                {
                    if (this.CubeBitm == null)
                        this.CubeBitm = GetShader(Map, br, bitmIndx, x, 0, 0, 0, 0, loadTextures);
                }
                else if (Map.Tags.TagsList[bitmIndx].Path.Contains(@"\detail_maps\")) // (\detail_maps\) - Can end in _bump though, thats why we put this first
                {
                    if (this.DetailBitm == null)
                        this.DetailBitm = GetShader(Map, br, bitmIndx, x, 0, 0, 0, 0, loadTextures);
                }
                else if (usage == 2) // (_bump)
                {
                    if (this.BumpBitm == null)
                        this.BumpBitm = GetShader(Map, br, bitmIndx, x, 0, 0, 0, 0, loadTextures);
                }
                else // Main texture
                {
                    if (this.MainBitm == null || this.MainBitm.TagPath.Contains(@"rasterizer\") || this.MainBitm.TagPath.Contains(@"default_"))
                        this.MainBitm = GetShader(Map, br, bitmIndx, x, 0, 0, 0, 0, loadTextures);
                }
            }

            #region Old Code
            /*
            int shadIndx = Map.tags.FindTagIndexByID(hr.ReadInt32());
            if (shadIndx == -1) return;

            ShaderName = Map.tags.tagsList[shadIndx].Path;
            hr.BaseStream.Position = Map.tags.tagsList[shadIndx].MetaOffset + 4;

            int stemIndx = Map.tags.FindTagIndexByID(hr.ReadInt32());
            Alpha = AlphaType.None;

            if (stemIndx != -1)
            {
                if (Map.tags.tagsList[stemIndx].Path.Contains("alphatest"))
                    Alpha = AlphaType.AlphaTest;

                else if (Map.tags.tagsList[stemIndx].Path.Contains("alpha")
                      || Map.tags.tagsList[stemIndx].Path.Contains("water"))
                    Alpha = AlphaType.AlphaBlend;
            }

            hr.BaseStream.Position = Map.tags.tagsList[shadIndx].MetaOffset + 12;

            int cc = hr.ReadInt32();
            int tr = hr.ReadInt32() - Map.index.secondaryMagic;

            if (cc != 0)
            {
                hr.BaseStream.Position = tr + 4;
                int bitmIndx = Map.tags.FindTagIndexByID(hr.ReadInt32());

                if (bitmIndx != -1)
                {
                    //this.MainTexture = HBitmap.LoadTexture(dev, hr, bitmIndx, 0, 0, 0, 0);
                    this.MainBitm = HBitmap.GetShader(hr, bitmIndx, 0, 0, 0, 0, loadBitm);
                }
            }

            hr.BaseStream.Position = Map.tags.tagsList[shadIndx].MetaOffset + 32;
            cc = hr.ReadInt32();
            tr = hr.ReadInt32() - Map.index.secondaryMagic;

            hr.BaseStream.Position = tr + 20;

            cc = hr.ReadInt32();
            int tr2 = hr.ReadInt32() - Map.index.secondaryMagic;

            bitmUVWScales = new BitmapUVWScale[cc];

            if (cc != 0)
            {
                for (int x = 0; x < cc; x++)
                {
                    hr.BaseStream.Position = tr2 + (x * 16);
                    bitmUVWScales[x] = new BitmapUVWScale(hr);
                }
            }

            hr.BaseStream.Position = tr + 4;
            cc = hr.ReadInt32();
            tr = hr.ReadInt32() - Map.index.secondaryMagic;

            if (cc != 0)
            {
                hr.BaseStream.Position = tr;
                int bitmIndx = Map.tags.FindTagIndexByID(hr.ReadInt32());

                if (bitmIndx != -1)
                {
                    bool reflectMaps = Map.tags.tagsList[bitmIndx].Path.Contains("reflection_maps");

                    if (Map.tags.tagsList[bitmIndx].Path.IndexOf("_bump") != -1)
                    {
                        //this.BumpMapTexture = HBitmap.LoadTexture(dev, hr, bitmIndx, 0, 0, 0, 0);
                        this.BumpBitm = HBitmap.GetShader(hr, bitmIndx, 0, 0, 0, 0, loadBitm);
                    }
                    else if (Map.tags.tagsList[bitmIndx].Path.IndexOf("_cube_map") != -1)
                    {
                        //Meta tempmeta = new Meta(tempcrap);
                        //tempmeta.ReadMetaFromMap(false, true, false, mapnum);
                        //ParsedBitmap pm = new ParsedBitmap(bitmIndx);

                        //this.CubeMapTexture = HBitmap.LoadTexture(dev, hr, bitmIndx, 0, 0, 0, 0);
                        this.CubeBitm = HBitmap.GetShader(hr, bitmIndx, 0, 0, 0, 0, loadBitm);
                    }
                    else if (Map.tags.tagsList[bitmIndx].Path.IndexOf("default_") == -1 &&
                        (this.MainBitm == null || !this.MainBitm.CanLoadBitmap) && !reflectMaps)
                    {
                        //Meta tempmeta = new Meta(tempcrap);
                        //tempmeta.ReadMetaFromMap(false, true, false, mapnum);
                        //ParsedBitmap pm = new ParsedBitmap(bitmIndx);

                        //this.MainTexture = HBitmap.LoadTexture(dev, hr, bitmIndx, 0, 0, 0, 0);
                        this.MainBitm = HBitmap.GetShader(hr, bitmIndx, 0, 0, 0, 0, loadBitm);
                    }
                    else if (reflectMaps)
                    {
                        hr.BaseStream.Position += 8;
                        bitmIndx = Map.tags.FindTagIndexByID(hr.ReadInt32());

                        //Meta tempmeta = new Meta(tempcrap);
                        //tempmeta.ReadMetaFromMap(false, true, false, mapnum);
                        //ParsedBitmap pm = new ParsedBitmap(bitmIndx);
                    }
                }

                hr.BaseStream.Position = tr + 24;
                bitmIndx = Map.tags.FindTagIndexByID(hr.ReadInt32());

                if (bitmIndx != -1 && (this.MainBitm == null || !this.MainBitm.CanLoadBitmap))
                {
                    //this.MainTexture = HBitmap.LoadTexture(dev, hr, bitmIndx, 0, 0, 0, 0);
                    this.MainBitm = HBitmap.GetShader(hr, bitmIndx, 0, 0, 0, 0, loadBitm);
                }
                else if ((this.MainBitm == null || !this.MainBitm.CanLoadBitmap))
                {
                    hr.BaseStream.Position = tr + 12;
                    bitmIndx = Map.tags.FindTagIndexByID(hr.ReadInt32());

                    if (bitmIndx != -1)
                    {
                        //this.MainTexture = HBitmap.LoadTexture(dev, hr, bitmIndx, 0, 0, 0, 0);
                        this.MainBitm = HBitmap.GetShader(hr, bitmIndx, 0, 0, 0, 0, loadBitm);
                    }
                }
            }
            */
            #endregion

            if (loadTextures)
            {
                CreateTextures();
                DisposeOf(true, false, false);
            }
        }

        public void WriteNewShaderIdent(int oldID, int newID)
        {
            BitReader br = new BitReader(Map.FileName, Endian.Little);
            BitWriter bw = new BitWriter(br.BaseStream, Endian.Little);

            Sbsp.BSPDeclaration bspD = Map.SBSP.BSPDeclarations[Map.SBSP.BSPIndex];
            br.BaseStream.Position = bspD.MetaOffset + 180;

            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - bspD.Magic;

            for (int x = 0; x < cc; x++)
            {
                br.BaseStream.Position = tr + (x * 32) + 12;
                if (br.ReadInt32() == oldID)
                {
                    bw.BaseStream.Position -= 4;
                    bw.Write(newID);
                }
            }

            bw.Close();
            br.Close();
        }

        public void WriteNewBitmapIdent(int bitmChunkIndx, int newID)
        {
            // Get shad index
            int shadIndx = Map.Tags.IndexOf(__identity__);
            if (shadIndx == -1) return;

            BitReader br = new BitReader(Map.FileName, Endian.Little);
            BitWriter bw = new BitWriter(br.BaseStream, Endian.Little);

            // Goto main reflexive
            br.BaseStream.Position = Map.Tags.TagsList[shadIndx].MetaOffset + 32;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            br.BaseStream.Position = tr + 4;

            // Bitmap inner-reflexive
            cc = br.ReadInt32();
            tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            bw.BaseStream.Position = tr + (bitmChunkIndx * 12);
            bw.Write(newID);

            bw.Close();
            br.Close();
        }

        public void DisposeOf(bool bitm, bool tex, bool info)
        {
            if (MainBitm != null) MainBitm.DisposeOf(bitm, tex);
            if (BumpBitm != null) BumpBitm.DisposeOf(bitm, tex);
            if (DetailBitm != null) DetailBitm.DisposeOf(bitm, tex);
            if (CubeBitm != null) CubeBitm.DisposeOf(bitm, tex);

            if (info)
            {
                MainBitm = null;
                BumpBitm = null;
                DetailBitm = null;
                CubeBitm = null;
            }
        }

        public static ShaderTextureInfo GetShader(Map map, int bitmTagIndx, int bitmChunkIndx, int submapIndx, int LOD, int paletteIndx, int sbspIndx, bool loadBitmaps)
        {
            BitReader br = new BitReader(map.FileName, Endian.Little);
            ShaderTextureInfo sInfo = GetShader(map, br, bitmTagIndx, bitmChunkIndx, submapIndx, LOD, paletteIndx, sbspIndx, loadBitmaps);
            br.Close();

            return sInfo;
        }

        public static ShaderTextureInfo GetShader(Map map, BitReader br, int bitmTagIndx, int bitmChunkIndx, int submapIndx, int LOD, int paletteIndx, int sbspIndx, bool loadBitmaps)
        {
            ShaderTextureInfo sInfo = new ShaderTextureInfo();

            sInfo.TagIdentity = map.Tags.TagsList[bitmTagIndx].Identity;
            sInfo.TagPath = map.Tags.TagsList[bitmTagIndx].Path;
            sInfo.ChunkIndex = bitmChunkIndx;

            br.BaseStream.Position = map.Tags.TagsList[bitmTagIndx].MetaOffset + 68;

            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - map.Index.SecondaryMagic;

            if (submapIndx >= cc) return null;
            br.BaseStream.Position = tr + (submapIndx * 116);

            sInfo.BitmInfo = new Bitm.BitmapInfo(br);
            br.BaseStream.Position += 4 + (LOD * 4);

            sInfo.RawOffset = br.ReadInt32();
            br.BaseStream.Position += 20;
            sInfo.RawSize = br.ReadInt32();

            if (sInfo.RawOffset == -1 || sInfo.RawSize == 0) return null;
            sInfo.ParsePointer(map);
            
            sInfo.CanLoadBitmap = true;
            if (loadBitmaps)
            {
                if (sInfo.RawLocation == RawLocation.Internal)
                {
                    br.BaseStream.Position = sInfo.RawOffset;
                    sInfo.Bitmap = Bitm.DecodeForBitmap(map, br.ReadBytes(sInfo.RawSize), sInfo.BitmInfo, paletteIndx, sbspIndx);
                }
                else
                {
                    FileStream fs = map.Prefs.GetRawStream(sInfo.RawLocation);

                    if (fs == null) sInfo.Bitmap = null;
                    else
                    {
                        BitReader rawBr = new BitReader(fs, Endian.Little);
                        rawBr.BaseStream.Position = sInfo.RawOffset;
                        sInfo.Bitmap = Bitm.DecodeForBitmap(map, rawBr.ReadBytes(sInfo.RawSize), sInfo.BitmInfo, paletteIndx, sbspIndx);

                        rawBr.Close();
                    }
                }
            }

            return sInfo;
        }

        public static Texture GetShaderTexture(Map map, ShaderTextureInfo sInfo)
        {
            BitReader br = new BitReader(map.FileName, Endian.Little);
            Texture tex = GetShaderTexture(map, br, sInfo);
            br.Close();

            return tex;
        }

        public static Texture GetShaderTexture(Map map, BitReader br, ShaderTextureInfo sInfo)
        {
            Bitmap bitm = null;

            if (sInfo.RawLocation == RawLocation.Internal)
            {
                br.BaseStream.Position = sInfo.RawOffset;
                bitm = Bitm.DecodeForBitmap(map, br.ReadBytes(sInfo.RawSize), sInfo.BitmInfo, 0, 0);
            }
            else
            {
                FileStream fs = map.Prefs.GetRawStreamRegardless(sInfo.RawLocation);
                if (fs == null) sInfo.Bitmap = null;
                else
                {
                    BitReader rawBr = new BitReader(fs, Endian.Little);
                    rawBr.BaseStream.Position = sInfo.RawOffset;
                    bitm = Bitm.DecodeForBitmap(map, rawBr.ReadBytes(sInfo.RawSize), sInfo.BitmInfo, 0, 0);

                    rawBr.Close();
                }
            }

            if (bitm == null) return null;
            else
            {
                Texture tex = CreateTexture(map.MDXFrameWork.Device, bitm);
                bitm.Dispose();
                return tex;
            }
        }

        public void CreateTextures()
        {
            if (MainBitm != null && MainBitm.Bitmap != null)
                MainBitm.Texture = CreateTexture(Map.MDXFrameWork.Device, MainBitm.Bitmap);

            //if (BumpBitm != null && BumpBitm.Bitmap != null)
            //    BumpBitm.Texture = CreateTexture(Map.MDXFrameWork.Device, BumpBitm.Bitmap);

            //if (DetailBitm != null && DetailBitm.Bitmap != null)
            //    DetailBitm.Texture = CreateTexture(Map.MDXFrameWork.Device, DetailBitm.Bitmap);

            //if (CubeBitm != null && CubeBitm.Bitmap != null)
            //    CubeBitm.CubeTexture = CreateCubeTexture(Map.MDXFrameWork.Device, CubeBitm.Bitmap);
        }

        public static Texture CreateTexture(Device dev, Bitmap bitm)
        {
            if (bitm == null) return null;

            Texture tex = new Texture(dev, bitm.Width, bitm.Height, 1, Usage.AutoGenerateMipMap,
                Format.A8R8G8B8, Pool.Managed);

            System.Drawing.Imaging.BitmapData bitmapData = bitm.LockBits(
                new Rectangle(0, 0, bitm.Width, bitm.Height), ImageLockMode.ReadOnly,
                PixelFormat.Format32bppArgb);

            int pitch;
            GraphicsStream textureData = tex.LockRectangle(0, LockFlags.None, out pitch);

            unsafe
            {
                int* texturePointer = (int*)textureData.InternalDataPointer;
                for (int y = 0; y < bitm.Height; y++)
                {
                    int* bitmapLinePointer = (int*)bitmapData.Scan0 + y * (bitmapData.Stride / sizeof(int));
                    int* textureLinePointer = texturePointer + y * (pitch / sizeof(int));
                    int length = bitm.Width;
                    while (--length >= 0) *textureLinePointer++ = *bitmapLinePointer++;
                }
            }

            bitm.UnlockBits(bitmapData);
            tex.UnlockRectangle(0);
            tex.GenerateMipSubLevels();
            tex.AutoGenerateFilterType = TextureFilter.Linear;
            return tex;
        }

        public static CubeTexture CreateCubeTexture(Device dev, Bitmap bitm)
        {
            if (bitm == null) return null;

            CubeTexture tex = new CubeTexture(dev, bitm.Width, 1, Usage.AutoGenerateMipMap,
                Format.A8R8G8B8, Pool.Managed);

            System.Drawing.Imaging.BitmapData bitmapData = bitm.LockBits(
                new Rectangle(0, 0, bitm.Width, bitm.Height), ImageLockMode.ReadOnly,
                PixelFormat.Format32bppArgb);

            int pitch;
            GraphicsStream textureData = tex.LockRectangle(0, 0, LockFlags.None, out pitch);

            unsafe
            {
                int* texturePointer = (int*)textureData.InternalDataPointer;
                for (int y = 0; y < bitm.Height; y++)
                {
                    int* bitmapLinePointer = (int*)bitmapData.Scan0 + y * (bitmapData.Stride / sizeof(int));
                    int* textureLinePointer = texturePointer + y * (pitch / sizeof(int));
                    int length = bitm.Width;
                    while (--length >= 0) *textureLinePointer++ = *bitmapLinePointer++;
                }
            }

            bitm.UnlockBits(bitmapData);
            tex.UnlockRectangle(0, 0);
            tex.GenerateMipSubLevels();
            tex.AutoGenerateFilterType = TextureFilter.Linear;
            return tex;
        }

        public static ShaderEditPanelTextureItem[] GetShaderInfoForShaderEditPanel(Map map, int shadIdentity)
        {
            // Get shad index
            int shadIndx = map.Tags.IndexOf(shadIdentity);
            if (shadIndx == -1) return null;

            BitReader br = new BitReader(map.FileName, Endian.Little);

            // Goto main reflexive
            br.BaseStream.Position = map.Tags.TagsList[shadIndx].MetaOffset + 32;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - map.Index.SecondaryMagic;

            br.BaseStream.Position = tr + 4;

            // Bitmap inner-reflexive
            cc = br.ReadInt32();
            tr = br.ReadInt32() - map.Index.SecondaryMagic;

            bool cubeBitmNull = true;
            bool detaBitmNull = true;
            bool bumpBitmNull = true;
            bool mainBitmNull = true;

            int cubeBitmIndex = -1;
            int detaBitmIndex = -1;
            int bumpBitmIndex = -1;
            int mainBitmIndex = -1;

            List<ShaderEditPanelTextureItem> items = new List<ShaderEditPanelTextureItem>();

            // Loop bitmaps
            for (int x = 0; x < cc; x++)
            {
                br.BaseStream.Position = tr + (x * 12);

                // Get bitmap index
                int bitmIndx = map.Tags.IndexOf(br.ReadInt32());
                if (bitmIndx == -1)
                {
                    items.Add(new ShaderEditPanelTextureItem(null, SEP_unknBitmString));
                    continue;
                }

                // Goto address0 in bitm tag
                br.BaseStream.Position = map.Tags.TagsList[bitmIndx].MetaOffset;

                // Enum16's
                int type = br.ReadInt16();
                int format = br.ReadInt16();
                int usage = br.ReadInt16();
                bool isUnk = false;

                if (type == 2) // (\reflection_maps\) OR (_cube_map)
                {
                    if (cubeBitmNull)
                    {
                        items.Add(new ShaderEditPanelTextureItem(
                            GetShader(map, br, bitmIndx, x, 0, 0, 0, 0, false), SEP_cubeBitmString));

                        cubeBitmIndex = items.Count - 1;
                        cubeBitmNull = false;
                    }
                    else isUnk = true;
                }
                else if (map.Tags.TagsList[bitmIndx].Path.Contains(@"\detail_maps\")) // (\detail_maps\) - Can end in _bump though, thats why we put this first
                {
                    if (detaBitmNull)
                    {
                        items.Add(new ShaderEditPanelTextureItem(
                            GetShader(map, br, bitmIndx, x, 0, 0, 0, 0, false), SEP_detaBitmString));

                        detaBitmIndex = items.Count - 1;
                        detaBitmNull = false;
                    }
                    else isUnk = true;
                }
                else if (usage == 2) // (_bump)
                {
                    if (bumpBitmNull)
                    {
                        items.Add(new ShaderEditPanelTextureItem(
                            GetShader(map, br, bitmIndx, x, 0, 0, 0, 0, false), SEP_bumpBitmString));

                        bumpBitmIndex = items.Count - 1;
                        bumpBitmNull = false;
                    }
                    else isUnk = true;
                }
                else
                {
                    // Main texture
                    if (mainBitmNull || items[mainBitmIndex].ShaderTextureInfo.TagPath.Contains(@"rasterizer\") || items[mainBitmIndex].ShaderTextureInfo.TagPath.Contains(@"default_"))
                    {
                        items.Add(new ShaderEditPanelTextureItem(
                            GetShader(map, br, bitmIndx, x, 0, 0, 0, 0, false), SEP_mainBitmString));

                        if (mainBitmIndex != -1) items[mainBitmIndex].DisplayName = SEP_unknBitmString;

                        mainBitmIndex = items.Count - 1;
                        mainBitmNull = false;
                    }
                    else isUnk = true;
                }

                if (isUnk)
                {
                    items.Add(new ShaderEditPanelTextureItem(
                        GetShader(map, br, bitmIndx, x, 0, 0, 0, 0, false), SEP_unknBitmString));
                }
            }
            
            ShaderEditPanelTextureItem[] temp = new ShaderEditPanelTextureItem[cc];
            int iNum = 0;

            if (mainBitmIndex != -1)
            {
                temp[iNum] = items[mainBitmIndex];
                iNum++;
            }
            if (bumpBitmIndex != -1)
            {
                temp[iNum] = items[bumpBitmIndex];
                iNum++;
            }
            if (detaBitmIndex != -1)
            {
                temp[iNum] = items[detaBitmIndex];
                iNum++;
            }
            if (cubeBitmIndex != -1)
            {
                temp[iNum] = items[cubeBitmIndex];
                iNum++;
            }
            
            for (int x = 0; x < items.Count; x++)
            {
                if (x != cubeBitmIndex &&
                    x != detaBitmIndex &&
                    x != bumpBitmIndex &&
                    x != mainBitmIndex)
                {
                    temp[iNum] = items[x];
                    iNum++;
                }
            }

            br.Close();
            return temp;
        }

        #endregion

        public enum AlphaType
        {
            None,
            AlphaTest,
            AlphaBlend
        }

        public struct BitmapUVWScale
        {
            #region Instance Data

            /// <summary>
            /// Uscale
            /// </summary>
            public float Uscale;
            /// <summary>
            /// Vscale
            /// </summary>
            public float Vscale;
            /// <summary>
            /// Wscale
            /// </summary>
            public float Wscale;
            /// <summary>
            /// Matrix
            /// </summary>
            //public Matrix Matrix { get { return __matrix__; } set { __matrix__ = value; } }

            #endregion

            public BitmapUVWScale(BitReader hr)
            {
                Uscale = hr.ReadSingle();
                Vscale = hr.ReadSingle();
                Wscale = hr.ReadSingle();
                //Matrix = Matrix.Scaling(Uscale, Vscale, Wscale);
            }
        }

        public class ShaderTextureInfo
        {
            #region Instance Data

            private int __tagIdentity__;
            private string __tagPath__;
            private int __chunkIndex__;
            private Bitm.BitmapInfo __bitmInfo__;
            private int __rawOffset__;
            private int __rawSize__;
            private RawLocation __rawLocation__;
            private bool __canLoadBitmap__;
            private Bitmap __bitmap__;
            private Texture __texture__;
            private CubeTexture __cubeTexture__;

            #endregion

            #region Accessors

            /// <summary>
            /// TagIdentity
            /// </summary>
            public int TagIdentity { get { return __tagIdentity__; } set { __tagIdentity__ = value; } }
            /// <summary>
            /// TagPath
            /// </summary>
            public string TagPath { get { return __tagPath__; } set { __tagPath__ = value; } }
            /// <summary>
            /// ChunkIndex
            /// </summary>
            public int ChunkIndex { get { return __chunkIndex__; } set { __chunkIndex__ = value; } }
            /// <summary>
            /// BitmInfo
            /// </summary>
            public Bitm.BitmapInfo BitmInfo { get { return __bitmInfo__; } set { __bitmInfo__ = value; } }
            /// <summary>
            /// RawOffset
            /// </summary>
            public int RawOffset { get { return __rawOffset__; } set { __rawOffset__ = value; } }
            /// <summary>
            /// RawSize
            /// </summary>
            public int RawSize { get { return __rawSize__; } set { __rawSize__ = value; } }
            /// <summary>
            /// RawLocation
            /// </summary>
            public RawLocation RawLocation { get { return __rawLocation__; } set { __rawLocation__ = value; } }
            /// <summary>
            /// CanLoadBitmap
            /// </summary>
            public bool CanLoadBitmap { get { return __canLoadBitmap__; } set { __canLoadBitmap__ = value; } }
            /// <summary>
            /// Bitmap
            /// </summary>
            public Bitmap Bitmap { get { return __bitmap__; } set { __bitmap__ = value; } }
            /// <summary>
            /// Texture
            /// </summary>
            public Texture Texture { get { return __texture__; } set { __texture__ = value; } }
            /// <summary>
            /// CubeTexture
            /// </summary>
            public CubeTexture CubeTexture { get { return __cubeTexture__; } set { __cubeTexture__ = value; } }

            #endregion

            public void ParsePointer(Map map)
            {
                map.Misc.ParsePointer(ref __rawOffset__, out __rawLocation__);
            }

            public int GetLODCount(Map map)
            {
                int tagIndx = map.Tags.IndexOf(TagIdentity);
                if (tagIndx == -1) return 0;

                BitReader br = new BitReader(map.FileName, Endian.Little);
                br.BaseStream.Position = map.Tags.TagsList[tagIndx].MetaOffset + 68 + 4;

                int tr = br.ReadInt32() - map.Index.SecondaryMagic;
                int submapIndx = 0;
                int count = 0;

                for (int y = 0; y < 6; y++)
                {
                    br.BaseStream.Position = tr + (submapIndx * 116) + 28 + (y * 4);

                    int rawOffset = br.ReadInt32();
                    br.BaseStream.Position += 20;
                    int rawSize = br.ReadInt32();

                    if (rawOffset != -1 && rawSize != 0)
                        count++;
                }

                br.Close();
                return count;
            }

            public void DisposeOf(bool bitm, bool tex)
            {
                if (bitm && this.Bitmap != null) this.Bitmap.Dispose();
                if (tex && this.Texture != null) this.Texture.Dispose();
                if (tex && this.CubeTexture != null) this.CubeTexture.Dispose();
            }
        }

        public class ShaderEditPanelTextureItem
        {
            #region Instance Data

            private ShaderTextureInfo __shaderTextureInfo__;
            private string __displayName__;

            #endregion

            #region Accessors

            /// <summary>
            /// ShaderTextureInfo
            /// </summary>
            public ShaderTextureInfo ShaderTextureInfo { get { return __shaderTextureInfo__; } set { __shaderTextureInfo__ = value; } }
            /// <summary>
            /// DisplayName
            /// </summary>
            public string DisplayName { get { return __displayName__; } set { __displayName__ = value; } }

            #endregion

            public ShaderEditPanelTextureItem(ShaderTextureInfo sInfo, string name)
            {
                ShaderTextureInfo = sInfo;
                DisplayName = name;
            }
        }
    }
}