//--------------------------------------------------------------------------------------
// File: DXMUTSettingsDlg.cs
//
// Dialog for selection of device settings 
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------
using System;
using System.IO;
using System.Collections;
using System.Runtime.InteropServices;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace MDXUtilityToolkit
{
    #region Control Ids
    public enum PrefsDialogControlIds
    {
        Static = -1,
        MemoryOption,
        ExternalMap,
        Spawn
    }
    #endregion

    /// <summary>
    /// Dialog for selection of device settings 
    /// </summary>
    public class PrefsDialog
    {
        /// <summary>Creates a new settings dialog</summary>
        public PrefsDialog(Prophet.Form1 sample)
        {
            parent = sample;
            windowWidth = MDXFramework.DefaultSizeWidth;
            windowHeight = MDXFramework.DefaultSizeHeight;
            CreateControls();
        }

        #region Class Data
        private Prophet.Form1 parent; // Parent framework for this dialog
        private Dialog dialog; // Dialog that will be rendered
        private uint windowWidth; // Width of window
        private uint windowHeight; // Height of window
        private DeviceSettings globalSettings; // Device settings
        private StateBlock state; // state block for device
        #endregion

        #region Control variables

        private ComboBox prefsComboBox;

        // Memory Eaters
        private StaticText BSP_Header;
        private Checkbox BSP_LoadShaders;
        private Checkbox BSP_LoadLightmaps;

        private StaticText Perms_Header;
        private Checkbox Perms_Load;
        private Checkbox Perms_LoadShaders;
        private Checkbox Perms_LoadLightmaps;

        // External Maps
        private StaticText mainmenuHeader;
        private EditBox mainmenuTextBox;
        private Button mainmenuButton;

        private StaticText mpsharedHeader;
        private EditBox mpsharedTextBox;
        private Button mpsharedButton;

        private StaticText spsharedHeader;
        private EditBox spsharedTextBox;
        private Button spsharedButton;

        /*
        // Spawns
        private Checkbox ShowBipedSpawns;
        private Checkbox ShowCameraSpawns;
        private Checkbox ShowCollectionSpawns;
        private Checkbox ShowControlSpawns;
        private Checkbox ShowDeathZoneSpawns;
        private Checkbox ShowEquipmentSpawns;
        private Checkbox ShowLightSpawnsSpawns;
        private Checkbox ShowMachineSpawns;
        private Checkbox ShowObjectiveSpawns;
        private Checkbox ShowObstacleSpawns;
        private Checkbox ShowPlayerSpawns;
        private Checkbox ShowScenerySpawns;
        private Checkbox ShowSpecialSpawns;
        private Checkbox ShowVehicleSpawns;
        private Checkbox ShowWeaponSpawns;

        private Button SpawnsCheckAllButton;
        private Button SpawnsUnCheckAllButton;
        */

        #endregion

        /// <summary>
        /// Creates the controls for use in the dialog
        /// </summary>
        private void CreateControls()
        {
            dialog = new Dialog(parent.Map.MDXFrameWork);
            dialog.IsUsingKeyboardInput = true;
            //dialog.SetFont(0, "Arial", 15, FontWeight.Normal);
            dialog.SetFont(1, "Arial", 28, FontWeight.Bold);
            //dialog.SetFont(2, "Arial", 16, FontWeight.Normal);
            //dialog.SetFont(3, "Comic Sans MS", 24, FontWeight.Normal);
            //dialog.SetFont(3, "Courier New", 16, FontWeight.Normal);

            // Right justify static controls
            Element e = dialog.GetDefaultElement(ControlType.StaticText, 0);
            e.textFormat = DrawTextFormat.VerticalCenter | DrawTextFormat.Left;

            // Title
            StaticText title = dialog.AddStatic((int)PrefsDialogControlIds.Static, "Preferences", 10, 5, 500, 30);
            title[0].FontIndex = 1;
            title[0].textFormat = DrawTextFormat.Top | DrawTextFormat.Left;

            // Combobox
            prefsComboBox = dialog.AddComboBox((int)PrefsDialogControlIds.Static, 80, 40, 300, 23);

            //Memory Options
            BSP_Header = dialog.AddStatic((int)PrefsDialogControlIds.MemoryOption, "BSP", 120, 65, 100, 23);
            BSP_LoadShaders = dialog.AddCheckBox((int)PrefsDialogControlIds.MemoryOption, "Load Shaders", 130, 90, 105, 23, parent.Map.Prefs.BSP_LoadShaders);
            BSP_LoadLightmaps = dialog.AddCheckBox((int)PrefsDialogControlIds.MemoryOption, "Load Lightmaps", 130, 115, 116, 23, parent.Map.Prefs.BSP_LoadLightmaps);

            Perms_Header = dialog.AddStatic((int)PrefsDialogControlIds.MemoryOption, "Permutations", 120, 150, 100, 23);
            Perms_Load = dialog.AddCheckBox((int)PrefsDialogControlIds.MemoryOption, "Load Permutations", 130, 175, 131, 23, parent.Map.Prefs.Perms_Load);
            Perms_LoadShaders = dialog.AddCheckBox((int)PrefsDialogControlIds.MemoryOption, "Load Shaders", 130, 200, 105, 23, parent.Map.Prefs.Perms_LoadShaders);
            Perms_LoadLightmaps = dialog.AddCheckBox((int)PrefsDialogControlIds.MemoryOption, "Load Lightmaps", 130, 225, 118, 23, parent.Map.Prefs.Perms_LoadLightmaps);

            SetMemoryOptionsVisibility(true);

            // External Maps
            mainmenuHeader = dialog.AddStatic((int)PrefsDialogControlIds.ExternalMap, "Mainmenu", 120, 74, 70, 23);
            mainmenuHeader[0].textFormat = DrawTextFormat.VerticalCenter | DrawTextFormat.Right;
            mainmenuTextBox = dialog.AddEditBox((int)PrefsDialogControlIds.ExternalMap, parent.Map.Prefs.MainMenuFileName, 200, 69, 250, 32);
            mainmenuButton = dialog.AddButton((int)PrefsDialogControlIds.ExternalMap, "...", 460, 69, 26, 31);

            mpsharedHeader = dialog.AddStatic((int)PrefsDialogControlIds.ExternalMap, "MP Shared", 120, 110, 70, 23);
            mpsharedHeader[0].textFormat = DrawTextFormat.VerticalCenter | DrawTextFormat.Right;
            mpsharedTextBox = dialog.AddEditBox((int)PrefsDialogControlIds.ExternalMap, parent.Map.Prefs.MPSharedFilename, 200, 105, 250, 32);
            mpsharedButton = dialog.AddButton((int)PrefsDialogControlIds.ExternalMap, "...", 460, 105, 26, 31);

            spsharedHeader = dialog.AddStatic((int)PrefsDialogControlIds.ExternalMap, "SP Shared", 120, 146, 70, 23);
            spsharedHeader[0].textFormat = DrawTextFormat.VerticalCenter | DrawTextFormat.Right;
            spsharedTextBox = dialog.AddEditBox((int)PrefsDialogControlIds.ExternalMap, parent.Map.Prefs.SPSharedFileName, 200, 141, 250, 32);
            spsharedButton = dialog.AddButton((int)PrefsDialogControlIds.ExternalMap, "...", 460, 141, 26, 31);

            SetExternalMapVisibility(false);

            /*
            // Spawns
            int ypos = 40;

            ShowBipedSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Bipeds", 120, ypos += 25, 105, 23, parent.Map.Prefs.ShowBipedSpawns);
            ShowCameraSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Cameras", 120, ypos += 25, 105, 23, parent.Map.Prefs.ShowCameraSpawns);
            ShowCollectionSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Collections", 120, ypos += 25, 105, 23, parent.Map.Prefs.ShowCollectionSpawns);
            ShowControlSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Controls", 120, ypos += 25, 105, 23, parent.Map.Prefs.ShowControlSpawns);
            ShowDeathZoneSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Death Zones", 120, ypos += 25, 105, 23, parent.Map.Prefs.ShowDeathZoneSpawns);
            ShowEquipmentSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Equipment", 120, ypos += 25, 105, 23, parent.Map.Prefs.ShowEquipmentSpawns);
            ShowLightSpawnsSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Lights", 120, ypos += 25, 105, 23, parent.Map.Prefs.ShowLightSpawnsSpawns);
            ShowMachineSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Machines", 120, ypos += 25, 105, 23, parent.Map.Prefs.ShowMachineSpawns);
            ShowObjectiveSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Objectives", 120, ypos += 25, 105, 23, parent.Map.Prefs.ShowObjectiveSpawns);

            SpawnsCheckAllButton = dialog.AddButton((int)PrefsDialogControlIds.Spawn, "Check All", 120, ypos += 30, 73, 31);
            SpawnsUnCheckAllButton = dialog.AddButton((int)PrefsDialogControlIds.Spawn, "Uncheck All", 200, ypos, 78, 31);

            ypos = 40;
            ShowObstacleSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Obstacles", 250, ypos += 25, 105, 23, parent.Map.Prefs.ShowObstacleSpawns);
            ShowPlayerSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Players", 250, ypos += 25, 105, 23, parent.Map.Prefs.ShowPlayerSpawns);
            ShowScenerySpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Scenery", 250, ypos += 25, 105, 23, parent.Map.Prefs.ShowScenerySpawns);
            ShowSpecialSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Special", 250, ypos += 25, 105, 23, parent.Map.Prefs.ShowSpecialSpawns);
            ShowVehicleSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Vehicles", 250, ypos += 25, 105, 23, parent.Map.Prefs.ShowVehicleSpawns);
            ShowWeaponSpawns = dialog.AddCheckBox((int)PrefsDialogControlIds.Spawn, "Weapons", 250, ypos += 25, 105, 23, parent.Map.Prefs.ShowWeaponSpawns);
            
            SetSpawnControls(false);
            */

            // Add the ok/cancel buttons
            Button okButton = dialog.AddButton((int)PrefsDialogControlIds.Static, "OK", 230, 350, 73, 31);
            Button cancelButton = dialog.AddButton((int)PrefsDialogControlIds.Static, "Cancel", 315, 350, 73, 31, 0, true);
            okButton.Click += new EventHandler(OnOkClicked);
            cancelButton.Click += new EventHandler(OnCancelClicked);
        }

        private void SetMemoryOptionsVisibility(bool vis)
        {
            //Memory Options
            BSP_Header.IsVisible = vis;
            BSP_LoadShaders.IsVisible = vis;
            BSP_LoadLightmaps.IsVisible = vis;

            Perms_Header.IsVisible = vis;
            Perms_Load.IsVisible = vis;
            Perms_LoadShaders.IsVisible = vis;
            Perms_LoadLightmaps.IsVisible = vis;
        }

        private void SetExternalMapVisibility(bool vis)
        {
            // External Maps
            mainmenuHeader.IsVisible = vis;
            mainmenuTextBox.IsVisible = vis;
            mainmenuButton.IsVisible = vis;

            mpsharedHeader.IsVisible = vis;
            mpsharedTextBox.IsVisible = vis;
            mpsharedButton.IsVisible = vis;

            spsharedHeader.IsVisible = vis;
            spsharedTextBox.IsVisible = vis;
            spsharedButton.IsVisible = vis;
        }

        private void SetSpawnControls(bool vis)
        {
            /*
            ShowBipedSpawns.IsVisible = vis;
            ShowCameraSpawns.IsVisible = vis;
            ShowCollectionSpawns.IsVisible = vis;
            ShowControlSpawns.IsVisible = vis;
            ShowDeathZoneSpawns.IsVisible = vis;
            ShowEquipmentSpawns.IsVisible = vis;
            ShowLightSpawnsSpawns.IsVisible = vis;
            ShowMachineSpawns.IsVisible = vis;
            ShowObjectiveSpawns.IsVisible = vis;
            ShowObstacleSpawns.IsVisible = vis;
            ShowPlayerSpawns.IsVisible = vis;
            ShowScenerySpawns.IsVisible = vis;
            ShowSpecialSpawns.IsVisible = vis;
            ShowVehicleSpawns.IsVisible = vis;
            ShowWeaponSpawns.IsVisible = vis;
            SpawnsCheckAllButton.IsVisible = vis;
            SpawnsUnCheckAllButton.IsVisible = vis;
            */
        }

        /// <summary>Changes the UI defaults to the current device settings</summary>
        public void Refresh()
        {
            // Get some information
            globalSettings = parent.Map.MDXFrameWork.DeviceSettings.Clone();
            System.Drawing.Rectangle client = parent.Map.MDXFrameWork.WindowClientRectangle;
            windowWidth = (uint)client.Width;
            windowHeight = (uint)client.Height;

            if (prefsComboBox.NumberItems == 0)
            {
                prefsComboBox.AddItem("Memory", 0);
                prefsComboBox.AddItem("External Maps", 1);
                //prefsComboBox.AddItem("Visible Spawns", 2);
            }

            // Memory Options
            BSP_LoadShaders.IsChecked = parent.Map.Prefs.BSP_LoadShaders;
            BSP_LoadLightmaps.IsChecked = parent.Map.Prefs.BSP_LoadLightmaps;

            Perms_Load.IsChecked = parent.Map.Prefs.Perms_Load;
            Perms_LoadShaders.IsChecked = parent.Map.Prefs.Perms_LoadShaders;
            Perms_LoadLightmaps.IsChecked = parent.Map.Prefs.Perms_LoadLightmaps;
            this.OnPermsLoadChanged(null, EventArgs.Empty);

            // External Maps
            mainmenuTextBox.SetText(parent.Map.Prefs.MainMenuFileName, false);
            mpsharedTextBox.SetText(parent.Map.Prefs.MPSharedFilename, false);
            spsharedTextBox.SetText(parent.Map.Prefs.SPSharedFileName, false);

            /*
            // Spawns
            ShowBipedSpawns.IsChecked = parent.Map.Prefs.ShowBipedSpawns;
            ShowCameraSpawns.IsChecked = parent.Map.Prefs.ShowCameraSpawns;
            ShowCollectionSpawns.IsChecked = parent.Map.Prefs.ShowCollectionSpawns;
            ShowControlSpawns.IsChecked = parent.Map.Prefs.ShowControlSpawns;
            ShowDeathZoneSpawns.IsChecked = parent.Map.Prefs.ShowDeathZoneSpawns;
            ShowEquipmentSpawns.IsChecked = parent.Map.Prefs.ShowEquipmentSpawns;
            ShowLightSpawnsSpawns.IsChecked = parent.Map.Prefs.ShowLightSpawnsSpawns;
            ShowMachineSpawns.IsChecked = parent.Map.Prefs.ShowMachineSpawns;
            ShowObjectiveSpawns.IsChecked = parent.Map.Prefs.ShowObjectiveSpawns;
            ShowObstacleSpawns.IsChecked = parent.Map.Prefs.ShowObstacleSpawns;
            ShowPlayerSpawns.IsChecked = parent.Map.Prefs.ShowPlayerSpawns;
            ShowScenerySpawns.IsChecked = parent.Map.Prefs.ShowScenerySpawns;
            ShowSpecialSpawns.IsChecked = parent.Map.Prefs.ShowSpecialSpawns;
            ShowVehicleSpawns.IsChecked = parent.Map.Prefs.ShowVehicleSpawns;
            ShowWeaponSpawns.IsChecked = parent.Map.Prefs.ShowWeaponSpawns;
            */

            Dialog.SetRefreshTime((float)FrameworkTimer.GetTime());
        }

        /// <summary>Render the dialog</summary>
        public void OnRender(float elapsedTime)
        {
            state.Capture();
            parent.Map.MDXFrameWork.Device.RenderState.FillMode = FillMode.Solid;
            dialog.OnRender(elapsedTime);
            state.Apply();
        }

        /// <summary>Hand messages off to dialog</summary>
        public void HandleMessages(IntPtr hWnd, NativeMethods.WindowMessage msg, IntPtr wParam, IntPtr lParam)
        {
            /*
            if (msg == NativeMethods.WindowMessage.KeyDown)
            {
                Control con = Perms_LoadLightmaps;

                System.Drawing.Size size = con.GetSize();
                con.SetSize(++size.Width, size.Height);

                BSP_LoadBitmaps.SetText(size.Width.ToString());
            }
             */

            dialog.MessageProc(hWnd, msg, wParam, lParam);
        }

        #region Device event callbacks

        /// <summary>
        /// Called when the device is created
        /// </summary>
        public void OnCreateDevice(Device d)
        {
            // Hook all the events we care about

            prefsComboBox.Changed += new EventHandler(OnPrefsComboBoxChanged);

            Perms_Load.Changed += new EventHandler(OnPermsLoadChanged);

            mainmenuButton.Click += new EventHandler(OnMainmenuButtonClicked);
            mpsharedButton.Click += new EventHandler(OnMPSharedButtonClicked);
            spsharedButton.Click += new EventHandler(OnSPSharedButtonClicked);

            //SpawnsCheckAllButton.Click += new EventHandler(OnSpawnsCheckAllButtonClick);
            //SpawnsUnCheckAllButton.Click += new EventHandler(OnSpawnsUnCheckAllButtonClick);
        }

        /// <summary>
        /// Called when the device is reset
        /// </summary>
        public void OnResetDevice()
        {
            SurfaceDescription desc = parent.Map.MDXFrameWork.BackBufferSurfaceDescription;

            // Set up the dialog
            dialog.SetBackgroundRect(new System.Drawing.Rectangle(0, 0, desc.Width, desc.Height));
            dialog.SetBackgroundColors(
                new ColorValue((float)98 / 255, (float)138 / 255, (float)206 / 255),
                new ColorValue((float)54 / 255, (float)105 / 255, (float)192 / 255),
                new ColorValue((float)54 / 255, (float)105 / 255, (float)192 / 255),
                new ColorValue((float)10 / 255, (float)73 / 255, (float)179 / 255));

            Device device = parent.Map.MDXFrameWork.Device;
            device.BeginStateBlock();
            device.RenderState.FillMode = FillMode.Solid;
            state = device.EndStateBlock();

            // Set up the dialog
            System.Drawing.Rectangle client = parent.Map.MDXFrameWork.WindowClientRectangle;
            windowWidth = (uint)client.Width;
            windowHeight = (uint)client.Height;

            System.Drawing.Size pSize = dialog.GetPreferredSize();
            dialog.SetSize(pSize.Width, pSize.Height);
            dialog.SetLocation(((int)windowWidth - pSize.Width) / 2, (((int)windowHeight - pSize.Height) / 2) - dialog.CaptionHeight);
        }

        /// <summary>
        /// Called when the device is lost
        /// </summary>
        public void OnLostDevice()
        {
            if (state != null)
                state.Dispose();

            state = null;
        }

        /// <summary>Destroy any resources</summary>
        public void OnDestroyDevice(object sender, EventArgs e)
        {
            // Clear the focus
            Dialog.ClearFocus();

            // Unhook all the events we care about
            prefsComboBox.Changed -= new EventHandler(OnPrefsComboBoxChanged);

            Perms_Load.Changed -= new EventHandler(OnPermsLoadChanged);

            mainmenuButton.Click -= new EventHandler(OnMainmenuButtonClicked);
            mpsharedButton.Click -= new EventHandler(OnMPSharedButtonClicked);
            spsharedButton.Click -= new EventHandler(OnSPSharedButtonClicked);

            //SpawnsCheckAllButton.Click -= new EventHandler(OnSpawnsCheckAllButtonClick);
            //SpawnsUnCheckAllButton.Click -= new EventHandler(OnSpawnsUnCheckAllButtonClick);
        }

        #endregion

        #region Event handlers for the controls

        private void OnPrefsComboBoxChanged(object sender, EventArgs e)
        {
            switch ((int)prefsComboBox.GetSelectedItem().ItemData)
            {
                case 0:
                    {
                        SetMemoryOptionsVisibility(true);
                        SetExternalMapVisibility(false);
                        //SetSpawnControls(false);
                        break;
                    }

                case 1:
                    {
                        SetMemoryOptionsVisibility(false);
                        SetExternalMapVisibility(true);
                        //SetSpawnControls(false);
                        break;
                    }
                /*
                case 2:
                    {
                        SetMemoryOptionsVisibility(false);
                        SetExternalMapVisibility(false);
                        SetSpawnControls(true);
                        break;
                    }
                */
            }
        }

        private void OnPermsLoadChanged(object sender, EventArgs e)
        {
            if (Perms_Load.IsChecked)
            {
                Perms_LoadShaders.IsEnabled = true;
                Perms_LoadLightmaps.IsEnabled = true;
            }
            else
            {
                Perms_LoadShaders.IsEnabled = false;
                Perms_LoadLightmaps.IsEnabled = false;

                Perms_LoadShaders.IsChecked = false;
                Perms_LoadLightmaps.IsChecked = false;
            }
        }

        private void OnMainmenuButtonClicked(object sender, EventArgs e)
        {
            parent.Map.MDXFrameWork.Pause(false, true);
            using (System.Windows.Forms.OpenFileDialog o = new System.Windows.Forms.OpenFileDialog())
            {
                o.Title = "\"mainmenu.map\" needed";
                o.Filter = "Map (*.map)|*.map|All Files (*.*)|*.*";
                if (o.ShowDialog() != System.Windows.Forms.DialogResult.Cancel)
                {
                    mainmenuTextBox.SetText(o.FileName, false);
                }
            }
            parent.Map.MDXFrameWork.Pause(false, false);
        }

        private void OnMPSharedButtonClicked(object sender, EventArgs e)
        {
            parent.Map.MDXFrameWork.Pause(false, true);
            using (System.Windows.Forms.OpenFileDialog o = new System.Windows.Forms.OpenFileDialog())
            {
                o.Title = "\"shared.map\" needed";
                o.Filter = "Map (*.map)|*.map|All Files (*.*)|*.*";
                if (o.ShowDialog() != System.Windows.Forms.DialogResult.Cancel)
                {
                    mpsharedTextBox.SetText(o.FileName, false);
                }
            }
            parent.Map.MDXFrameWork.Pause(false, false);
        }

        private void OnSPSharedButtonClicked(object sender, EventArgs e)
        {
            parent.Map.MDXFrameWork.Pause(false, true);
            using (System.Windows.Forms.OpenFileDialog o = new System.Windows.Forms.OpenFileDialog())
            {
                o.Title = "\"single_player_shared.map\" needed";
                o.Filter = "Map (*.map)|*.map|All Files (*.*)|*.*";
                if (o.ShowDialog() != System.Windows.Forms.DialogResult.Cancel)
                {
                    spsharedTextBox.SetText(o.FileName, false);
                }
            }
            parent.Map.MDXFrameWork.Pause(false, false);
        }

        private void OnSpawnsCheckAllButtonClick(object sender, EventArgs e)
        {
            /*
            ShowBipedSpawns.IsChecked = true;
            ShowCameraSpawns.IsChecked = true;
            ShowCollectionSpawns.IsChecked = true;
            ShowControlSpawns.IsChecked = true;
            ShowDeathZoneSpawns.IsChecked = true;
            ShowEquipmentSpawns.IsChecked = true;
            ShowLightSpawnsSpawns.IsChecked = true;
            ShowMachineSpawns.IsChecked = true;
            ShowObjectiveSpawns.IsChecked = true;
            ShowObstacleSpawns.IsChecked = true;
            ShowPlayerSpawns.IsChecked = true;
            ShowScenerySpawns.IsChecked = true;
            ShowSpecialSpawns.IsChecked = true;
            ShowVehicleSpawns.IsChecked = true;
            ShowWeaponSpawns.IsChecked = true;
            */
        }

        private void OnSpawnsUnCheckAllButtonClick(object sender, EventArgs e)
        {
            /*
            ShowBipedSpawns.IsChecked = false;
            ShowCameraSpawns.IsChecked = false;
            ShowCollectionSpawns.IsChecked = false;
            ShowControlSpawns.IsChecked = false;
            ShowDeathZoneSpawns.IsChecked = false;
            ShowEquipmentSpawns.IsChecked = false;
            ShowLightSpawnsSpawns.IsChecked = false;
            ShowMachineSpawns.IsChecked = false;
            ShowObjectiveSpawns.IsChecked = false;
            ShowObstacleSpawns.IsChecked = false;
            ShowPlayerSpawns.IsChecked = false;
            ShowScenerySpawns.IsChecked = false;
            ShowSpecialSpawns.IsChecked = false;
            ShowVehicleSpawns.IsChecked = false;
            ShowWeaponSpawns.IsChecked = false;
            */
        }

        /// <summary>Called when the cancel button is clicked</summary>
        private void OnCancelClicked(object sender, EventArgs e)
        {
            // Nothing left to do, quit showing the screen
            parent.Map.MDXFrameWork.RenderPage = RenderingPages.Default;
        }

        /// <summary>Called when the ok button is clicked</summary>
        private void OnOkClicked(object sender, EventArgs e)
        {
            bool refreshNeeded = false;

            // Mem. stuffz
            if (parent.Map.Prefs.BSP_LoadShaders != BSP_LoadShaders.IsChecked) refreshNeeded = true;
            parent.Map.Prefs.BSP_LoadShaders = BSP_LoadShaders.IsChecked;

            if (parent.Map.Prefs.BSP_LoadLightmaps != BSP_LoadLightmaps.IsChecked) refreshNeeded = true;
            parent.Map.Prefs.BSP_LoadLightmaps = BSP_LoadLightmaps.IsChecked;

            if (parent.Map.Prefs.Perms_Load != Perms_Load.IsChecked) refreshNeeded = true;
            parent.Map.Prefs.Perms_Load = Perms_Load.IsChecked;

            if (parent.Map.Prefs.Perms_LoadShaders != Perms_LoadShaders.IsChecked) refreshNeeded = true;
            parent.Map.Prefs.Perms_LoadShaders = Perms_LoadShaders.IsChecked;

            if (parent.Map.Prefs.Perms_LoadLightmaps != Perms_LoadLightmaps.IsChecked) refreshNeeded = true;
            parent.Map.Prefs.Perms_LoadLightmaps = Perms_LoadLightmaps.IsChecked;

            // External maps
            if (parent.Map.Prefs.MainMenuFileName != mainmenuTextBox.Text) refreshNeeded = true;
            parent.Map.Prefs.MainMenuFileName = mainmenuTextBox.Text;

            if (parent.Map.Prefs.MPSharedFilename != mpsharedTextBox.Text) refreshNeeded = true;
            parent.Map.Prefs.MPSharedFilename = mpsharedTextBox.Text;

            if (parent.Map.Prefs.SPSharedFileName != spsharedTextBox.Text) refreshNeeded = true;
            parent.Map.Prefs.SPSharedFileName = spsharedTextBox.Text;

            /*
            // Spawns
            parent.Map.Prefs.ShowBipedSpawns = ShowBipedSpawns.IsChecked;
            parent.Map.Prefs.ShowCameraSpawns = ShowCameraSpawns.IsChecked;
            parent.Map.Prefs.ShowCollectionSpawns = ShowCollectionSpawns.IsChecked;
            parent.Map.Prefs.ShowControlSpawns = ShowControlSpawns.IsChecked;
            parent.Map.Prefs.ShowDeathZoneSpawns = ShowDeathZoneSpawns.IsChecked;
            parent.Map.Prefs.ShowEquipmentSpawns = ShowEquipmentSpawns.IsChecked;
            parent.Map.Prefs.ShowLightSpawnsSpawns = ShowLightSpawnsSpawns.IsChecked;
            parent.Map.Prefs.ShowMachineSpawns = ShowMachineSpawns.IsChecked;
            parent.Map.Prefs.ShowObjectiveSpawns = ShowObjectiveSpawns.IsChecked;
            parent.Map.Prefs.ShowObstacleSpawns = ShowObstacleSpawns.IsChecked;
            parent.Map.Prefs.ShowPlayerSpawns = ShowPlayerSpawns.IsChecked;
            parent.Map.Prefs.ShowScenerySpawns = ShowScenerySpawns.IsChecked;
            parent.Map.Prefs.ShowSpecialSpawns = ShowSpecialSpawns.IsChecked;
            parent.Map.Prefs.ShowVehicleSpawns = ShowVehicleSpawns.IsChecked;
            parent.Map.Prefs.ShowWeaponSpawns = ShowWeaponSpawns.IsChecked;
            */

            // Stop showing the dialog now
            parent.Map.MDXFrameWork.RenderPage = RenderingPages.Default;

            if (refreshNeeded) parent.LoadForRenderWrapper(false, true);
        }

        #endregion
    }
}